@@ -2,9 +2,10 @@
 /* The name of this package (not localized) */
 const PACKAGE_NAME = 'gnome-shell';
 /* The version of this package */
-const PACKAGE_VERSION = '3.1.91';
+const PACKAGE_VERSION = '3.1.91.1';
 /* The version of GJS we're linking to */
 const GJS_VERSION = '1.29.17';
 /* 1 if gnome-bluetooth is available, 0 otherwise */
 const HAVE_BLUETOOTH = 1;
-
+/* The system TLS CA list */
+const SHELL_SYSTEM_CA_FILE = '/etc/pki/tls/certs/ca-bundle.crt';
