@@ -45,17 +45,6 @@ const NM80211ApSecurityFlags = NetworkManager['80211ApSecurityFlags'];
 // (the remaining are placed into More...)
 const NUM_VISIBLE_NETWORKS = 5;
 
-const NMAppletHelperInterface = {
-    name: 'org.gnome.network_manager_applet',
-    methods: [
-        { name: 'ConnectToHiddenNetwork', inSignature: '', outSignature: '' },
-        { name: 'CreateWifiNetwork', inSignature: '', outSignature: '' },
-        { name: 'ConnectTo8021xNetwork', inSignature: 'oo', outSignature: '' },
-        { name: 'ConnectTo3gNetwork', inSignature: 'o', outSignature: '' }
-    ],
-};
-const NMAppletProxy = DBus.makeProxyClass(NMAppletHelperInterface);
-
 function macToArray(string) {
     return string.split(':').map(function(el) {
         return parseInt(el, 16);
@@ -102,6 +91,13 @@ function sortAccessPoints(accessPoints) {
     });
 }
 
+function ssidToLabel(ssid) {
+    let label = NetworkManager.utils_ssid_to_utf8(ssid);
+    if (!label)
+        label = _("<unknown>");
+    return label;
+}
+
 function NMNetworkMenuItem() {
     this._init.apply(this, arguments);
 }
@@ -117,10 +113,7 @@ NMNetworkMenuItem.prototype = {
 
         if (!title) {
             let ssid = this.bestAP.get_ssid();
-            if (ssid)
-                title = NetworkManager.utils_ssid_to_utf8(ssid);
-            if (!title)
-                title = _("<unknown>");
+            title = ssidToLabel(ssid);
         }
 
         this._label = new St.Label({ text: title });
@@ -631,15 +624,8 @@ NMDevice.prototype = {
             this.emit('network-lost');
         }
 
-        switch(newstate) {
-        case NetworkManager.DeviceState.NEED_AUTH:
-            // FIXME: make this have a real effect
-            // (currently we rely on a running nm-applet)
-            this.emit('need-auth');
-            break;
-        case NetworkManager.DeviceState.FAILED:
+        if (newstate == NetworkManager.DeviceState.FAILED) {
             this.emit('activation-failed', reason);
-            break;
         }
 
         this._updateStatusItem();
@@ -674,7 +660,7 @@ NMDevice.prototype = {
         let dev_product = this.device.get_product();
         let dev_vendor = this.device.get_vendor();
         if (!dev_product || !dev_vendor)
-	    return null;
+	    return '';
 
         let product = Util.fixupPCIDescription(dev_product);
         let vendor = Util.fixupPCIDescription(dev_vendor);
@@ -748,10 +734,6 @@ NMDeviceModem.prototype = {
         this.mobileDevice = null;
         this._connectionType = 'ppp';
 
-        this._applet_proxy = new NMAppletProxy(DBus.session,
-                                               'org.gnome.network_manager_applet',
-                                               '/org/gnome/network_manager_applet');
-
         this._capabilities = device.current_capabilities;
         if (this._capabilities & NetworkManager.DeviceModemCapabilities.GSM_UMTS) {
             is_wwan = true;
@@ -853,12 +835,10 @@ NMDeviceModem.prototype = {
     },
 
     _createAutomaticConnection: function() {
-        // Mobile wizard is handled by nm-applet for now...
-        this._applet_proxy.ConnectTo3gNetworkRemote(this.device.get_path(),
-                                                    Lang.bind(this, function(results, err) {
-                                                        if (err)
-                                                            log(err);
-                                                    }));
+        // Mobile wizard is too complex for the shell UI and
+        // is handled by the network panel
+        Util.spawn(['gnome-control-center', 'network',
+                    'connect-3g', this.device.get_path()]);
         return null;
     }
 };
@@ -970,10 +950,6 @@ NMDeviceWireless.prototype = {
         this._overflowItem = null;
         this._networks = [ ];
 
-        this._applet_proxy = new NMAppletProxy(DBus.session,
-                                               'org.gnome.network_manager_applet',
-                                               '/org/gnome/network_manager_applet');
-
         // breaking the layers with this, but cannot call
         // this.connectionValid until I have a device
         this.device = device;
@@ -985,6 +961,16 @@ NMDeviceWireless.prototype = {
         for (let i = 0; i < accessPoints.length; i++) {
             // Access points are grouped by network
             let ap = accessPoints[i];
+
+            if (ap.get_ssid() == null) {
+                // hidden access point cannot be added, we need to know
+                // the SSID and security details to connect
+                // nevertheless, the access point can acquire a SSID when
+                // NetworkManager connects to it (via nmcli or the control-center)
+                ap._notifySsidId = ap.connect('notify::ssid', Lang.bind(this, this._notifySsidCb));
+                continue;
+            }
+
             let pos = this._findNetwork(ap);
             let obj;
             if (pos != -1) {
@@ -998,7 +984,7 @@ NMDeviceWireless.prototype = {
                         item: null,
                         accessPoints: [ ap ]
                       };
-                obj.ssidText = NetworkManager.utils_ssid_to_utf8(obj.ssid);
+                obj.ssidText = ssidToLabel(obj.ssid);
                 this._networks.push(obj);
             }
 
@@ -1011,8 +997,14 @@ NMDeviceWireless.prototype = {
                 }
             }
         }
+
         if (this.device.active_access_point) {
-            this._activeNetwork = this._networks[this._findNetwork(this.device.active_access_point)];
+            let networkPos = this._findNetwork(this.device.active_access_point);
+
+            if (networkPos == -1) // the connected access point is invisible
+                this._activeNetwork = null;
+            else
+                this._activeNetwork = this._networks[networkPos];
         } else {
             this._activeNetwork = null;
         }
@@ -1098,6 +1090,14 @@ NMDeviceWireless.prototype = {
         }
     },
 
+    _notifySsidCb: function(accessPoint) {
+        if (accessPoint.get_ssid() != null) {
+            accessPoint.disconnect(accessPoint._notifySsidId);
+            accessPoint._notifySsidId = 0;
+            this._accessPointAdded(this.device, accessPoint);
+        }
+    },
+
     _activeApChanged: function() {
         this._activeNetwork = null;
 
@@ -1105,7 +1105,9 @@ NMDeviceWireless.prototype = {
 
         if (activeAp) {
             let pos = this._findNetwork(activeAp);
-            this._activeNetwork = this._networks[pos];
+
+            if (pos != -1)
+                this._activeNetwork = this._networks[pos];
         }
 
         // we don't refresh the view here, setActiveConnection will
@@ -1180,6 +1182,9 @@ NMDeviceWireless.prototype = {
     },
 
     _findNetwork: function(accessPoint) {
+        if (accessPoint.get_ssid() == null)
+            return -1;
+
         for (let i = 0; i < this._networks.length; i++) {
             if (this._networkCompare(this._networks[i], accessPoint))
                 return i;
@@ -1188,6 +1193,13 @@ NMDeviceWireless.prototype = {
     },
 
     _accessPointAdded: function(device, accessPoint) {
+        if (accessPoint.get_ssid() == null) {
+            // This access point is not visible yet
+            // Wait for it to get a ssid
+            accessPoint._notifySsidId = accessPoint.connect('notify::ssid', Lang.bind(this, this._notifySsidCb));
+            return;
+        }
+
         let pos = this._findNetwork(accessPoint);
         let apObj;
         let needsupdate = false;
@@ -1210,7 +1222,7 @@ NMDeviceWireless.prototype = {
                       item: null,
                       accessPoints: [ accessPoint ]
                     };
-            apObj.ssidText = NetworkManager.utils_ssid_to_utf8(apObj.ssid);
+            apObj.ssidText = ssidToLabel(apObj.ssid);
             needsupdate = true;
         }
 
@@ -1423,13 +1435,12 @@ NMDeviceWireless.prototype = {
     },
 
     _createActiveConnectionItem: function() {
-        let activeAp = this.device.active_access_point;
         let icon, title;
         if (this._activeConnection._connection) {
             let connection = this._activeConnection._connection;
-            if (activeAp)
-                this._activeConnectionItem = new NMNetworkMenuItem([ activeAp ], undefined,
-                                                                       { reactive: false });
+            if (this._activeNetwork)
+                this._activeConnectionItem = new NMNetworkMenuItem(this._activeNetwork.accessPoints, undefined,
+                                                                   { reactive: false });
             else
                 this._activeConnectionItem = new PopupMenu.PopupImageMenuItem(connection._name,
                                                                               'network-wireless-connected',
@@ -1437,9 +1448,9 @@ NMDeviceWireless.prototype = {
         } else {
             // We cannot read the connection (due to ACL, or API incompatibility), but we still show signal if we have it
             let menuItem;
-            if (activeAp)
-                this._activeConnectionItem = new NMNetworkMenuItem([ activeAp ], undefined,
-                                                                       { reactive: false });
+            if (this._activeNetwork)
+                this._activeConnectionItem = new NMNetworkMenuItem(this._activeNetwork.accessPoints, undefined,
+                                                                   { reactive: false });
             else
                 this._activeConnectionItem = new PopupMenu.PopupImageMenuItem(_("Connected (private)"),
                                                                               'network-wireless-connected',
@@ -1486,13 +1497,10 @@ NMDeviceWireless.prototype = {
                 let accessPoints = sortAccessPoints(apObj.accessPoints);
                 if (   (accessPoints[0]._secType == NMAccessPointSecurity.WPA2_ENT)
                     || (accessPoints[0]._secType == NMAccessPointSecurity.WPA_ENT)) {
-                    // 802.1x-enabled APs get handled by nm-applet for now...
-                    this._applet_proxy.ConnectTo8021xNetworkRemote(this.device.get_path(),
-                                                                   accessPoints[0].dbus_path,
-                                                                   Lang.bind(this, function(results, err) {
-                                                                       if (err)
-                                                                           log(err);
-                                                                   }));
+                    // 802.1x-enabled APs require further configuration, so they're
+                    // handled in gnome-control-center
+                    Util.spawn(['gnome-control-center', 'network', 'connect-8021x-wifi',
+                                this.device.get_path(), accessPoints[0].dbus_path]);
                 } else {
                     let connection = this._createAutomaticConnection(apObj);
                     this._client.add_and_activate_connection(connection, this.device, accessPoints[0].dbus_path, null)
@@ -1550,9 +1558,9 @@ NMApplet.prototype = {
         this._statusSection.addAction(_("Enable networking"), Lang.bind(this, function() {
             this._client.networking_enabled = true;
         }));
-        this._statusSection.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._statusSection.actor.hide();
         this.menu.addMenuItem(this._statusSection);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
         this._devices = { };
 
@@ -1563,9 +1571,9 @@ NMApplet.prototype = {
         };
 
         this._devices.wired.section.addMenuItem(this._devices.wired.item);
-        this._devices.wired.section.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._devices.wired.section.actor.hide();
         this.menu.addMenuItem(this._devices.wired.section);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
         this._devices.wireless = {
             section: new PopupMenu.PopupMenuSection(),
@@ -1573,9 +1581,9 @@ NMApplet.prototype = {
             item: this._makeToggleItem('wireless', _("Wireless"))
         };
         this._devices.wireless.section.addMenuItem(this._devices.wireless.item);
-        this._devices.wireless.section.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._devices.wireless.section.actor.hide();
         this.menu.addMenuItem(this._devices.wireless.section);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
         this._devices.wwan = {
             section: new PopupMenu.PopupMenuSection(),
@@ -1583,9 +1591,9 @@ NMApplet.prototype = {
             item: this._makeToggleItem('wwan', _("Mobile broadband"))
         };
         this._devices.wwan.section.addMenuItem(this._devices.wwan.item);
-        this._devices.wwan.section.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._devices.wwan.section.actor.hide();
         this.menu.addMenuItem(this._devices.wwan.section);
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
         this._devices.vpn = {
             section: new PopupMenu.PopupMenuSection(),
@@ -1598,15 +1606,10 @@ NMApplet.prototype = {
         this._devices.vpn.item.updateForDevice(this._devices.vpn.device);
         this._devices.vpn.section.addMenuItem(this._devices.vpn.item);
         this._devices.vpn.section.addMenuItem(this._devices.vpn.device.section);
-        this._devices.vpn.section.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
         this._devices.vpn.section.actor.hide();
         this.menu.addMenuItem(this._devices.vpn.section);
-
-        this.menu.addAction(_("Network Settings"), function() {
-            Main.overview.hide();
-            let app = Shell.AppSystem.get_default().get_app('gnome-network-panel.desktop');
-            app.activate(-1);
-        });
+        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
+        this.menu.addSettingsAction(_("Network Settings"), 'gnome-network-panel.desktop');
 
         this._activeConnections = [ ];
         this._connections = [ ];
@@ -1660,8 +1663,7 @@ NMApplet.prototype = {
     _ensureSource: function() {
         if (!this._source) {
             this._source = new NMMessageTraySource();
-            this._source._destroyId = this._source.connect('destroy', Lang.bind(this, function() {
-                this._source._destroyId = 0;
+            this._source.connect('destroy', Lang.bind(this, function() {
                 this._source = null;
             }));
             Main.messageTray.add(this._source);
@@ -1709,6 +1711,28 @@ NMApplet.prototype = {
         }
     },
 
+    _notifyForDevice: function(device, iconName, title, text, urgency) {
+        if (device._notification)
+            device._notification.destroy();
+
+        /* must call after destroying previous notification,
+           or this._source will be cleared */
+        this._ensureSource();
+
+        let icon = new St.Icon({ icon_name: iconName,
+                                 icon_type: St.IconType.SYMBOLIC,
+                                 icon_size: this._source.ICON_SIZE
+                               });
+        device._notification = new MessageTray.Notification(this._source, title, text,
+                                                            { icon: icon });
+        device._notification.setUrgency(urgency);
+        device._notification.setTransient(true);
+        device._notification.connect('destroy', function() {
+            device._notification = null;
+        });
+        this._source.notify(device._notification);
+    },
+
     _deviceAdded: function(client, device) {
         if (device._delegate) {
             // already seen, not adding again
@@ -1718,42 +1742,29 @@ NMApplet.prototype = {
         if (wrapperClass) {
             let wrapper = new wrapperClass(this._client, device, this._connections);
 
-            // FIXME: these notifications are duplicate with those exposed by nm-applet
-            // uncomment this code in 3.2, when we'll conflict with and kill nm-applet
-            /* wrapper._networkLostId = wrapper.connect('network-lost', Lang.bind(this, function(emitter) {
-                this._ensureSource();
-                let icon = new St.Icon({ icon_name: 'network-offline',
-                                         icon_type: St.IconType.SYMBOLIC,
-                                         icon_size: this._source.ICON_SIZE
-                                       });
-                let notification = new MessageTray.Notification(this._source,
-                                                                _("Connectivity lost"),
-                                                                _("You're no longer connected to the network"),
-                                                                { icon: icon });
-                this._source.notify(notification);
+            wrapper._networkLostId = wrapper.connect('network-lost', Lang.bind(this, function(device) {
+                this._notifyForDevice(device, 'network-offline',
+                                      _("Connectivity lost"),
+                                      _("You are no longer connected to the network"),
+                                      // set critical urgency to popup the notification automatically
+                                      MessageTray.Urgency.CRITICAL);
             }));
-            wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(wrapper, reason) {
-                this._ensureSource();
-                let icon = new St.Icon({ icon_name: 'network-error',
-                                         icon_type: St.IconType.SYMBOLIC,
-                                         icon_size: this._source.ICON_SIZE,
-                                       });
-                let banner;
+            wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(device, reason) {
                 // XXX: nm-applet has no special text depending on reason
                 // but I'm not sure of this generic message
-                let notification = new MessageTray.Notification(this._source,
-                                                                _("Connection failed"),
-                                                                _("Activation of network connection failed"),
-                                                                { icon: icon });
-                this._source.notify(notification);
-            })); */
+                this._notifyForDevice(device, 'network-error',
+                                      _("Connection failed"),
+                                      _("Activation of network connection failed"),
+                                     MessageTray.Urgency.HIGH);
+            }));
             wrapper._deviceStateChangedId = wrapper.connect('state-changed', Lang.bind(this, function(dev) {
                 this._syncSectionTitle(dev.category);
             }));
             wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
-                //wrapper.disconnect(wrapper._networkLostId);
-                //wrapper.disconnect(wrapper._activationFailedId);
+                wrapper.disconnect(wrapper._networkLostId);
+                wrapper.disconnect(wrapper._activationFailedId);
                 wrapper.disconnect(wrapper._deviceStateChangedId);
+                wrapper.disconnect(wrapper._destroyId);
             });
             let section = this._devices[wrapper.category].section;
             let devices = this._devices[wrapper.category].devices;
@@ -1798,11 +1809,8 @@ NMApplet.prototype = {
                 active._primaryDevice.setActiveConnection(null);
                 active._primaryDevice = null;
             }
-            if (active._notifyStateId) {
-                active.disconnect(active._notifyStateId);
-                active._notifyStateId = 0;
-            }
             if (active._inited) {
+                active.disconnect(active._notifyStateId);
                 active.disconnect(active._notifyDefaultId);
                 active.disconnect(active._notifyDefault6Id);
                 active._inited = false;
@@ -1820,14 +1828,7 @@ NMApplet.prototype = {
             if (!a._inited) {
                 a._notifyDefaultId = a.connect('notify::default', Lang.bind(this, this._updateIcon));
                 a._notifyDefault6Id = a.connect('notify::default6', Lang.bind(this, this._updateIcon));
-                if (a.state == NetworkManager.ActiveConnectionState.ACTIVATING) // prepare to notify to the user
-                    a._notifyStateId = a.connect('notify::state', Lang.bind(this, this._notifyActiveConnection));
-                else {
-                    // notify as soon as possible
-                    Mainloop.idle_add(Lang.bind(this, function() {
-                        this._notifyActiveConnection(a);
-                    }));
-                }
+                a._notifyStateId = a.connect('notify::state', Lang.bind(this, this._notifyActivated));
 
                 a._inited = true;
             }
@@ -1871,67 +1872,26 @@ NMApplet.prototype = {
 
                 if (a._primaryDevice)
                     a._primaryDevice.setActiveConnection(a);
+
+                if (a.state == NetworkManager.ActiveConnectionState.ACTIVATED
+                    && a._primaryDevice && a._primaryDevice._notification) {
+                    a._primaryDevice._notification.destroy();
+                    a._primaryDevice._notification = null;
+                }
             }
         }
 
         this._mainConnection = activating || default_ip4 || default_ip6 || this._activeConnections[0] || null;
     },
 
-    _notifyActiveConnection: function(active) {
-        // FIXME: duplicate notifications when nm-applet is running
-        // This code will come back when nm-applet is killed
-        this._syncNMState();
-        return;
-
-        if (active.state == NetworkManager.ActiveConnectionState.ACTIVATED) {
-
-            // notify only connections that are visible
-            if (active._connection) {
-                this._ensureSource();
-
-                let icon;
-                let banner;
-                switch (active._section) {
-                case NMConnectionCategory.WWAN:
-                    icon = 'network-cellular-signal-excellent';
-                    banner = _("You're now connected to mobile broadband connection '%s'").format(active._connection._name);
-                    break;
-                case NMConnectionCategory.WIRELESS:
-                    icon = 'network-wireless-signal-excellent';
-                    banner = _("You're now connected to wireless network '%s'").format(active._connection._name);
-                    break;
-                case NMConnectionCategory.WIRED:
-                    icon = 'network-wired';
-                    banner = _("You're now connected to wired network '%s'").format(active._connection._name);
-                    break;
-                case NMConnectionCategory.VPN:
-                    icon = 'network-vpn';
-                    banner = _("You're now connected to VPN network '%s'").format(active._connection._name);
-                    break;
-                default:
-                    // a fallback for a generic 'connected' icon
-                    icon = 'network-transmit-receive';
-                    banner = _("You're now connected to '%s'").format(active._connection._name);
-                }
-
-                let iconActor = new St.Icon({ icon_name: icon,
-                                              icon_type: St.IconType.SYMBOLIC,
-                                              icon_size: this._source.ICON_SIZE
-                                            });
-                let notification = new MessageTray.Notification(this._source,
-                                                                _("Connection established"),
-                                                                banner,
-                                                                { icon: iconActor });
-                this._source.notify(notification);
-            }
-
-            if (active._stateChangeId) {
-                active.disconnect(active._stateChangeId);
-                active._stateChangeId = 0;
-            }
+    _notifyActivated: function(activeConnection) {
+        if (activeConnection.state == NetworkManager.ActiveConnectionState.ACTIVATED
+            && activeConnection._primaryDevice && activeConnection._primaryDevice._notification) {
+            activeConnection._primaryDevice._notification.destroy();
+            activeConnection._primaryDevice._notification = null;
         }
 
-        this._syncNMState();
+        this._updateIcon();
     },
 
     _readConnections: function() {
