@@ -170,6 +170,7 @@ IMStatusChooserItem.prototype = {
         this._presence.connect('StatusChanged',
                                Lang.bind(this, this._sessionStatusChanged));
 
+        this._currentPresence = undefined;
         this._previousPresence = undefined;
 
         this._accountMgr = Tp.AccountManager.dup()
@@ -226,6 +227,7 @@ IMStatusChooserItem.prototype = {
 
     _setIconFromFile: function(iconFile) {
         this._iconBin.set_style('background-image: url("' + iconFile + '");');
+        this._iconBin.child = null;
     },
 
     _setIconFromName: function(iconName) {
@@ -249,23 +251,28 @@ IMStatusChooserItem.prototype = {
     _statusForPresence: function(presence) {
         switch(presence) {
             case Tp.ConnectionPresenceType.AVAILABLE:
-                return _("Available");
+                return 'available';
             case Tp.ConnectionPresenceType.BUSY:
-                return _("Busy");
+                return 'busy';
             case Tp.ConnectionPresenceType.OFFLINE:
-                return _("Unavailable");
+                return 'offline';
             case Tp.ConnectionPresenceType.HIDDEN:
-                return _("Hidden");
+                return 'hidden';
             case Tp.ConnectionPresenceType.AWAY:
-                return _("Away");
+                return 'away';
             case Tp.ConnectionPresenceType.EXTENDED_AWAY:
-                return _("Idle");
+                return 'xa';
             default:
-                return _("Unknown");
+                return 'unknown';
         }
     },
 
     _IMStatusChanged: function(accountMgr, presence, status, message) {
+        if (presence == this._currentPresence)
+            return;
+
+        this._currentPresence = presence;
+
         if (presence == Tp.ConnectionPresenceType.AVAILABLE)
             this._presence.setStatus(GnomeSession.PresenceStatus.AVAILABLE);
 
@@ -515,8 +522,8 @@ UserMenuButton.prototype = {
     },
 
     _updateSwitch: function(presence, status) {
-        let active = status == GnomeSession.PresenceStatus.BUSY;
-        this._dontDisturbSwitch.setToggleState(active);
+        let active = status == GnomeSession.PresenceStatus.AVAILABLE;
+        this._notificationsSwitch.setToggleState(active);
     },
 
     _updatePresenceIcon: function(accountMgr, presence, status, message) {
@@ -541,10 +548,10 @@ UserMenuButton.prototype = {
         item.connect('activate', Lang.bind(this, this._onMyAccountActivate));
         this.menu.addMenuItem(item);
 
-        item = new PopupMenu.PopupSwitchMenuItem(_("Do Not Disturb"));
+        item = new PopupMenu.PopupSwitchMenuItem(_("Notifications"));
         item.connect('activate', Lang.bind(this, this._updatePresenceStatus));
         this.menu.addMenuItem(item);
-        this._dontDisturbSwitch = item;
+        this._notificationsSwitch = item;
 
         item = new PopupMenu.PopupSeparatorMenuItem();
         this.menu.addMenuItem(item);
@@ -587,8 +594,8 @@ UserMenuButton.prototype = {
     },
 
     _updatePresenceStatus: function(item, event) {
-        let status = item.state ? GnomeSession.PresenceStatus.BUSY
-                                : GnomeSession.PresenceStatus.AVAILABLE;
+        let status = item.state ? GnomeSession.PresenceStatus.AVAILABLE
+                                : GnomeSession.PresenceStatus.BUSY;
         this._presence.setStatus(status);
     },
 
