@@ -11,9 +11,19 @@
 #include "shell-app-private.h"
 #include "shell-enum-types.h"
 #include "shell-global.h"
+#include "shell-util.h"
+#include "shell-app-system-private.h"
 #include "shell-window-tracker-private.h"
 #include "st.h"
 
+typedef enum {
+  MATCH_NONE,
+  MATCH_SUBSTRING, /* Not prefix, substring */
+  MATCH_MULTIPLE_SUBSTRING, /* Matches multiple criteria with substrings */
+  MATCH_PREFIX, /* Strict prefix */
+  MATCH_MULTIPLE_PREFIX, /* Matches multiple criteria, at least one prefix */
+} ShellAppSearchMatch;
+
 /* This is mainly a memory usage optimization - the user is going to
  * be running far fewer of the applications at one time than they have
  * installed.  But it also just helps keep the code more logically
@@ -38,7 +48,7 @@ typedef struct {
  * SECTION:shell-app
  * @short_description: Object representing an application
  *
- * This object wraps a #ShellAppInfo, providing methods and signals
+ * This object wraps a #GMenuTreeEntry, providing methods and signals
  * primarily useful for running applications.
  */
 struct _ShellApp
@@ -49,9 +59,22 @@ struct _ShellApp
 
   ShellAppState state;
 
-  ShellAppInfo *info;
+  GMenuTreeEntry *entry; /* If NULL, this app is backed by one or more
+                          * MetaWindow.  For purposes of app title
+                          * etc., we use the first window added,
+                          * because it's most likely to be what we
+                          * want (e.g. it will be of TYPE_NORMAL from
+                          * the way shell-window-tracker.c works).
+                          */
 
   ShellAppRunningState *running_state;
+
+  char *window_id_string;
+
+  char *casefolded_name;
+  char *name_collation_key;
+  char *casefolded_description;
+  char *casefolded_exec;
 };
 
 G_DEFINE_TYPE (ShellApp, shell_app, G_TYPE_OBJECT);
@@ -93,7 +116,44 @@ shell_app_get_property (GObject    *gobject,
 const char *
 shell_app_get_id (ShellApp *app)
 {
-  return shell_app_info_get_id (app->info);
+  if (app->entry)
+    return gmenu_tree_entry_get_desktop_file_id (app->entry);
+  return app->window_id_string;
+}
+
+static MetaWindow *
+window_backed_app_get_window (ShellApp     *app)
+{
+  g_assert (app->entry == NULL);
+  g_assert (app->running_state);
+  g_assert (app->running_state->windows);
+  return app->running_state->windows->data;
+}
+
+static ClutterActor *
+window_backed_app_get_icon (ShellApp *app,
+                            int       size)
+{
+  MetaWindow *window;
+  ClutterActor *actor;
+
+  /* During a state transition from running to not-running for
+   * window-backend apps, it's possible we get a request for the icon.
+   * Avoid asserting here and just return an empty image.
+   */
+  if (app->running_state == NULL)
+    {
+      actor = clutter_texture_new ();
+      g_object_set (actor, "opacity", 0, "width", (float) size, "height", (float) size, NULL);
+      return actor;
+    }
+
+  window = window_backed_app_get_window (app);
+  actor = st_texture_cache_bind_pixbuf_property (st_texture_cache_get_default (),
+                                                               G_OBJECT (window),
+                                                               "icon");
+  g_object_set (actor, "width", (float) size, "height", (float) size, NULL);
+  return actor;
 }
 
 /**
@@ -106,10 +166,30 @@ shell_app_get_id (ShellApp *app)
  */
 ClutterActor *
 shell_app_create_icon_texture (ShellApp   *app,
-                               float size)
+                               int         size)
 {
-  return shell_app_info_create_icon_texture (app->info, size);
+  GIcon *icon;
+  ClutterActor *ret;
+
+  ret = NULL;
+
+  if (app->entry == NULL)
+    return window_backed_app_get_icon (app, size);
+
+  icon = g_app_info_get_icon (G_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
+  if (icon != NULL)
+    ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), NULL, icon, size);
+
+  if (ret == NULL)
+    {
+      icon = g_themed_icon_new ("application-x-executable");
+      ret = st_texture_cache_load_gicon (st_texture_cache_get_default (), NULL, icon, size);
+      g_object_unref (icon);
+    }
+
+  return ret;
 }
+
 typedef struct {
   ShellApp *app;
   int size;
@@ -143,20 +223,19 @@ shell_app_create_faded_icon_cpu (StTextureCache *cache,
 
   info = NULL;
 
-  icon = shell_app_info_get_icon (app->info);
+  icon = g_app_info_get_icon (G_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
   if (icon != NULL)
     {
       info = gtk_icon_theme_lookup_by_gicon (gtk_icon_theme_get_default (),
-                                             icon, (int) (size + 0.5),
+                                             icon, size,
                                              GTK_ICON_LOOKUP_FORCE_SIZE);
-      g_object_unref (icon);
     }
 
   if (info == NULL)
     {
       icon = g_themed_icon_new ("application-x-executable");
       info = gtk_icon_theme_lookup_by_gicon (gtk_icon_theme_get_default (),
-                                             icon, (int) (size + 0.5),
+                                             icon, size,
                                              GTK_ICON_LOOKUP_FORCE_SIZE);
       g_object_unref (icon);
     }
@@ -222,28 +301,23 @@ shell_app_create_faded_icon_cpu (StTextureCache *cache,
  * Return value: (transfer none): A floating #ClutterActor, or %NULL if no icon
  */
 ClutterActor *
-shell_app_get_faded_icon (ShellApp *app, float size)
+shell_app_get_faded_icon (ShellApp *app, int size)
 {
-  MetaWindow *window;
   CoglHandle texture;
   ClutterActor *result;
   char *cache_key;
   CreateFadedIconData data;
 
-  /* Punt for WINDOW types for now...easier to reuse the property tracking bits,
-   * and this helps us visually distinguish app-tracked from not.
+  /* Don't fade for window backed apps for now...easier to reuse the
+   * property tracking bits, and this helps us visually distinguish
+   * app-tracked from not.
    */
-  window = shell_app_info_get_source_window (app->info);
-  if (window)
-    {
-      return st_texture_cache_bind_pixbuf_property (st_texture_cache_get_default (),
-                                                    G_OBJECT (window),
-                                                    "icon");
-    }
+  if (!app->entry)
+    return window_backed_app_get_icon (app, size);
 
-  cache_key = g_strdup_printf ("faded-icon:%s,size=%f", shell_app_get_id (app), size);
+  cache_key = g_strdup_printf ("faded-icon:%s,size=%d", shell_app_get_id (app), size);
   data.app = app;
-  data.size = (int) (0.5 + size);
+  data.size = size;
   texture = st_texture_cache_load (st_texture_cache_get_default (),
                                    cache_key,
                                    ST_TEXTURE_CACHE_POLICY_FOREVER,
@@ -260,28 +334,49 @@ shell_app_get_faded_icon (ShellApp *app, float size)
   else
     {
       result = clutter_texture_new ();
-      g_object_set (result, "opacity", 0, "width", size, "height", size, NULL);
+      g_object_set (result, "opacity", 0, "width", (float) size, "height", (float) size, NULL);
 
     }
   return result;
 }
 
-char *
+const char *
 shell_app_get_name (ShellApp *app)
 {
-  return shell_app_info_get_name (app->info);
+  if (app->entry)
+    return g_app_info_get_name (G_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
+  else
+    {
+      MetaWindow *window = window_backed_app_get_window (app);
+      const char *name;
+
+      name = meta_window_get_wm_class (window);
+      if (!name)
+        name = _("Unknown");
+      return name;
+    }
 }
 
-char *
+const char *
 shell_app_get_description (ShellApp *app)
 {
-  return shell_app_info_get_description (app->info);
+  if (app->entry)
+    return g_app_info_get_description (G_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
+  else
+    return NULL;
 }
 
+/**
+ * shell_app_is_window_backed:
+ *
+ * A window backed application is one which represents just an open
+ * window, i.e. there's no .desktop file assocation, so we don't know
+ * how to launch it again.
+ */
 gboolean
-shell_app_is_transient (ShellApp *app)
+shell_app_is_window_backed (ShellApp *app)
 {
-  return shell_app_info_is_transient (app->info);
+  return app->entry == NULL;
 }
 
 typedef struct {
@@ -415,6 +510,16 @@ shell_app_activate_window (ShellApp     *app,
                                                   meta_window_get_user_time (most_recent_transient)))
         window = most_recent_transient;
 
+
+      if (!shell_window_tracker_is_window_interesting (window))
+        {
+          /* We won't get notify::user-time signals for uninteresting windows,
+           * which means that an app's last_user_time won't get updated.
+           * Update it here instead.
+           */
+          app->running_state->last_user_time = timestamp;
+        }
+
       if (active != workspace)
         meta_workspace_activate_with_focus (workspace, window, timestamp);
       else
@@ -425,8 +530,22 @@ shell_app_activate_window (ShellApp     *app,
 /**
  * shell_app_activate:
  * @app: a #ShellApp
+ *
+ * Like shell_app_activate_full(), but using the default workspace and
+ * event timestamp.
+ */
+void
+shell_app_activate (ShellApp      *app)
+{
+  return shell_app_activate_full (app, -1, 0);
+}
+
+/**
+ * shell_app_activate_full:
+ * @app: a #ShellApp
  * @workspace: launch on this workspace, or -1 for default. Ignored if
  *   activating an existing window
+ * @timestamp: Event timestamp
  *
  * Perform an appropriate default action for operating on this application,
  * dependent on its current state.  For example, if the application is not
@@ -435,24 +554,32 @@ shell_app_activate_window (ShellApp     *app,
  * recently used transient for that window).
  */
 void
-shell_app_activate (ShellApp      *app,
-                    int            workspace)
+shell_app_activate_full (ShellApp      *app,
+                         int            workspace,
+                         guint32        timestamp)
 {
+  ShellGlobal *global;
+
+  global = shell_global_get ();
+
+  if (timestamp == 0)
+    timestamp = shell_global_get_current_time (global);
+
   switch (app->state)
     {
       case SHELL_APP_STATE_STOPPED:
         {
           GError *error = NULL;
-          if (!shell_app_info_launch_full (app->info,
-                                           0,
-                                           NULL,
-                                           workspace,
-                                           NULL,
-                                           &error))
+          if (!shell_app_launch (app,
+                                 timestamp,
+                                 NULL,
+                                 workspace,
+                                 NULL,
+                                 &error))
             {
               char *msg;
               msg = g_strdup_printf (_("Failed to launch '%s'"), shell_app_get_name (app));
-              shell_global_notify_error (shell_global_get (),
+              shell_global_notify_error (global,
                                          msg,
                                          error->message);
               g_free (msg);
@@ -463,7 +590,7 @@ shell_app_activate (ShellApp      *app,
       case SHELL_APP_STATE_STARTING:
         break;
       case SHELL_APP_STATE_RUNNING:
-        shell_app_activate_window (app, NULL, shell_global_get_current_time (shell_global_get ()));
+        shell_app_activate_window (app, NULL, timestamp);
         break;
     }
 }
@@ -479,6 +606,8 @@ void
 shell_app_open_new_window (ShellApp      *app,
                            int            workspace)
 {
+  g_return_if_fail (app->entry != NULL);
+
   /* Here we just always launch the application again, even if we know
    * it was already running.  For most applications this
    * should have the effect of creating a new window, whether that's
@@ -487,12 +616,12 @@ shell_app_open_new_window (ShellApp      *app,
    * as say Pidgin.  Ideally, we have the application express to us
    * that it supports an explicit new-window action.
    */
-  shell_app_info_launch_full (app->info,
-                              0,
-                              NULL,
-                              workspace,
-                              NULL,
-                              NULL);
+  shell_app_launch (app,
+                    0,
+                    NULL,
+                    workspace,
+                    NULL,
+                    NULL);
 }
 
 /**
@@ -507,17 +636,6 @@ shell_app_get_state (ShellApp *app)
   return app->state;
 }
 
-/**
- * _shell_app_get_info:
- *
- * Returns: (transfer none): Associated app info
- */
-ShellAppInfo *
-_shell_app_get_info (ShellApp *app)
-{
-  return app->info;
-}
-
 typedef struct {
   ShellApp *app;
   MetaWorkspace *active_workspace;
@@ -687,21 +805,22 @@ _shell_app_new_for_window (MetaWindow      *window)
   ShellApp *app;
 
   app = g_object_new (SHELL_TYPE_APP, NULL);
-  app->info = shell_app_system_create_from_window (shell_app_system_get_default (), window);
-  _shell_app_system_register_app (shell_app_system_get_default (), app);
+
+  app->window_id_string = g_strdup_printf ("window:%d", meta_window_get_stable_sequence (window));
+
   _shell_app_add_window (app, window);
 
   return app;
 }
 
 ShellApp *
-_shell_app_new (ShellAppInfo    *info)
+_shell_app_new (GMenuTreeEntry *info)
 {
   ShellApp *app;
 
   app = g_object_new (SHELL_TYPE_APP, NULL);
-  app->info = shell_app_info_ref (info);
-  _shell_app_system_register_app (shell_app_system_get_default (), app);
+  app->entry = gmenu_tree_item_ref (info);
+  app->name_collation_key = g_utf8_collate_key (shell_app_get_name (app), -1);
 
   return app;
 }
@@ -722,7 +841,7 @@ shell_app_state_transition (ShellApp      *app,
       app->running_state = NULL;
     }
 
-  _shell_window_tracker_notify_app_state_changed (shell_window_tracker_get_default (), app);
+  _shell_app_system_notify_app_state_changed (shell_app_system_get_default (), app);
 
   g_object_notify (G_OBJECT (app), "state");
 }
@@ -912,6 +1031,112 @@ shell_app_request_quit (ShellApp   *app)
   return TRUE;
 }
 
+static void
+_gather_pid_callback (GDesktopAppInfo   *gapp,
+                      GPid               pid,
+                      gpointer           data)
+{
+  ShellApp *app;
+  ShellWindowTracker *tracker;
+
+  g_return_if_fail (data != NULL);
+
+  app = SHELL_APP (data);
+  tracker = shell_window_tracker_get_default ();
+
+  _shell_window_tracker_add_child_process_app (tracker,
+                                               pid,
+                                               app);
+}
+
+/**
+ * shell_app_launch:
+ * @timestamp: Event timestamp, or 0 for current event timestamp
+ * @uris: (element-type utf8): List of uris to pass to application
+ * @workspace: Start on this workspace, or -1 for default
+ * @startup_id: (out): Returned startup notification ID, or %NULL if none
+ * @error: A #GError
+ */
+gboolean
+shell_app_launch (ShellApp     *app,
+                  guint         timestamp,
+                  GList        *uris,
+                  int           workspace,
+                  char        **startup_id,
+                  GError      **error)
+{
+  GDesktopAppInfo *gapp;
+  GdkAppLaunchContext *context;
+  gboolean ret;
+  ShellGlobal *global;
+  MetaScreen *screen;
+
+  if (startup_id)
+    *startup_id = NULL;
+
+  if (app->entry == NULL)
+    {
+      MetaWindow *window = window_backed_app_get_window (app);
+      /* We can't pass URIs into a window; shouldn't hit this
+       * code path.  If we do, fix the caller to disallow it.
+       */
+      g_return_val_if_fail (uris == NULL, TRUE);
+
+      meta_window_activate (window, timestamp);
+      return TRUE;
+    }
+
+  global = shell_global_get ();
+  screen = shell_global_get_screen (global);
+
+  if (timestamp == 0)
+    timestamp = shell_global_get_current_time (global);
+
+  if (workspace < 0)
+    workspace = meta_screen_get_active_workspace_index (screen);
+
+  context = gdk_app_launch_context_new ();
+  gdk_app_launch_context_set_timestamp (context, timestamp);
+  gdk_app_launch_context_set_desktop (context, workspace);
+
+  gapp = gmenu_tree_entry_get_app_info (app->entry);
+  ret = g_desktop_app_info_launch_uris_as_manager (gapp, uris,
+                                                   G_APP_LAUNCH_CONTEXT (context),
+                                                   G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+                                                   NULL, NULL,
+                                                   _gather_pid_callback, app,
+                                                   error);
+  g_object_unref (context);
+
+  return ret;
+}
+
+/**
+ * shell_app_get_app_info:
+ * @app: a #ShellApp
+ *
+ * Returns: (transfer none): The #GDesktopAppInfo for this app, or %NULL if backed by a window
+ */
+GDesktopAppInfo *
+shell_app_get_app_info (ShellApp *app)
+{
+  if (app->entry)
+    return gmenu_tree_entry_get_app_info (app->entry);
+  return NULL;
+}
+
+/**
+ * shell_app_get_tree_entry:
+ * @app: a #ShellApp
+ *
+ * Returns: (transfer none): The #GMenuTreeEntry for this app, or %NULL if backed by a window
+ */
+GMenuTreeEntry *
+shell_app_get_tree_entry (ShellApp *app)
+{
+  return app->entry;
+}
+
 static void
 create_running_state (ShellApp *app)
 {
@@ -941,6 +1166,157 @@ unref_running_state (ShellAppRunningState *state)
   g_slice_free (ShellAppRunningState, state);
 }
 
+static char *
+trim_exec_line (const char *str)
+{
+  const char *start, *end, *pos;
+
+  end = strchr (str, ' ');
+  if (end == NULL)
+    end = str + strlen (str);
+
+  start = str;
+  while ((pos = strchr (start, '/')) && pos < end)
+    start = ++pos;
+
+  return g_strndup (start, end - start);
+}
+
+static void
+shell_app_init_search_data (ShellApp *app)
+{
+  const char *name;
+  const char *exec;
+  const char *comment;
+  char *normalized_exec;
+  GDesktopAppInfo *appinfo;
+
+  appinfo = gmenu_tree_entry_get_app_info (app->entry);
+  name = g_app_info_get_name (G_APP_INFO (appinfo));
+  app->casefolded_name = shell_util_normalize_and_casefold (name);
+
+  comment = g_app_info_get_description (G_APP_INFO (appinfo));
+  app->casefolded_description = shell_util_normalize_and_casefold (comment);
+
+  exec = g_app_info_get_executable (G_APP_INFO (appinfo));
+  normalized_exec = shell_util_normalize_and_casefold (exec);
+  app->casefolded_exec = trim_exec_line (normalized_exec);
+  g_free (normalized_exec);
+}
+
+/**
+ * shell_app_compare_by_name:
+ * @app:
+ * @other:
+ *
+ * Order two applications by name.
+ *
+ * Returns: -1, 0, or 1; suitable for use as a comparison function for e.g. g_slist_sort()
+ */
+int
+shell_app_compare_by_name (ShellApp *app, ShellApp *other)
+{
+  return strcmp (app->name_collation_key, other->name_collation_key);
+}
+
+static ShellAppSearchMatch
+_shell_app_match_search_terms (ShellApp  *app,
+                               GSList    *terms)
+{
+  GSList *iter;
+  ShellAppSearchMatch match;
+
+  if (G_UNLIKELY (!app->casefolded_name))
+    shell_app_init_search_data (app);
+
+  match = MATCH_NONE;
+  for (iter = terms; iter; iter = iter->next)
+    {
+      ShellAppSearchMatch current_match;
+      const char *term = iter->data;
+      const char *p;
+
+      current_match = MATCH_NONE;
+
+      p = strstr (app->casefolded_name, term);
+      if (p == app->casefolded_name)
+        current_match = MATCH_PREFIX;
+      else if (p != NULL)
+        current_match = MATCH_SUBSTRING;
+
+      p = strstr (app->casefolded_exec, term);
+      if (p != NULL)
+        {
+          if (p == app->casefolded_exec)
+            current_match = (current_match == MATCH_NONE) ? MATCH_PREFIX
+                                                          : MATCH_MULTIPLE_PREFIX;
+          else if (current_match < MATCH_PREFIX)
+            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
+                                                          : MATCH_MULTIPLE_SUBSTRING;
+        }
+
+      if (app->casefolded_description && current_match < MATCH_PREFIX)
+        {
+          /* Only do substring matches, as prefix matches are not meaningful
+           * enough for descriptions
+           */
+          p = strstr (app->casefolded_description, term);
+          if (p != NULL)
+            current_match = (current_match == MATCH_NONE) ? MATCH_SUBSTRING
+                                                          : MATCH_MULTIPLE_SUBSTRING;
+        }
+
+      if (current_match == MATCH_NONE)
+        return current_match;
+
+      if (current_match > match)
+        match = current_match;
+    }
+  return match;
+}
+
+void
+_shell_app_do_match (ShellApp         *app,
+                     GSList           *terms,
+                     GSList          **multiple_prefix_results,
+                     GSList          **prefix_results,
+                     GSList          **multiple_substring_results,
+                     GSList          **substring_results)
+{
+  ShellAppSearchMatch match;
+  GAppInfo *appinfo;
+
+  g_assert (app != NULL);
+
+  /* Skip window-backed apps */ 
+  appinfo = (GAppInfo*)shell_app_get_app_info (app);
+  if (appinfo == NULL)
+    return;
+  /* Skip not-visible apps */ 
+  if (!g_app_info_should_show (appinfo))
+    return;
+
+  match = _shell_app_match_search_terms (app, terms);
+  switch (match)
+    {
+      case MATCH_NONE:
+        break;
+      case MATCH_MULTIPLE_PREFIX:
+        *multiple_prefix_results = g_slist_prepend (*multiple_prefix_results, app);
+        break;
+      case MATCH_PREFIX:
+        *prefix_results = g_slist_prepend (*prefix_results, app);
+        break;
+      case MATCH_MULTIPLE_SUBSTRING:
+        *multiple_substring_results = g_slist_prepend (*multiple_substring_results, app);
+        break;
+      case MATCH_SUBSTRING:
+        *substring_results = g_slist_prepend (*substring_results, app);
+        break;
+    }
+}
+
+
 static void
 shell_app_init (ShellApp *self)
 {
@@ -952,10 +1328,10 @@ shell_app_dispose (GObject *object)
 {
   ShellApp *app = SHELL_APP (object);
 
-  if (app->info)
+  if (app->entry)
     {
-      shell_app_info_unref (app->info);
-      app->info = NULL;
+      gmenu_tree_item_unref (app->entry);
+      app->entry = NULL;
     }
 
   if (app->running_state)
@@ -967,6 +1343,20 @@ shell_app_dispose (GObject *object)
   G_OBJECT_CLASS(shell_app_parent_class)->dispose (object);
 }
 
+static void
+shell_app_finalize (GObject *object)
+{
+  ShellApp *app = SHELL_APP (object);
+
+  g_free (app->window_id_string);
+
+  g_free (app->casefolded_name);
+  g_free (app->name_collation_key);
+  g_free (app->casefolded_description);
+
+  G_OBJECT_CLASS(shell_app_parent_class)->finalize (object);
+}
+
 static void
 shell_app_class_init(ShellAppClass *klass)
 {
@@ -974,6 +1364,7 @@ shell_app_class_init(ShellAppClass *klass)
 
   gobject_class->get_property = shell_app_get_property;
   gobject_class->dispose = shell_app_dispose;
+  gobject_class->finalize = shell_app_finalize;
 
   shell_app_signals[WINDOWS_CHANGED] = g_signal_new ("windows-changed",
                                      SHELL_TYPE_APP,
