@@ -6,6 +6,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <math.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -17,7 +18,6 @@
 #include <canberra.h>
 #include <clutter/glx/clutter-glx.h>
 #include <clutter/x11/clutter-x11.h>
-#include <dbus/dbus-glib.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
 #include <gjs/gjs-module.h>
@@ -45,7 +45,19 @@ static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data
 
 struct _ShellGlobal {
   GObject parent;
-  
+
+  ClutterStage *stage;
+  Window stage_xwindow;
+  GdkWindow *stage_gdk_window;
+
+  MetaDisplay *meta_display;
+  GdkDisplay *gdk_display;
+  Display *xdisplay;
+  MetaScreen *meta_screen;
+  GdkScreen *gdk_screen;
+
+  ShellSessionType session_type;
+
   /* We use this window to get a notification from GTK+ when
    * a widget in our process does a GTK+ grab.  See
    * http://bugzilla.gnome.org/show_bug.cgi?id=570641
@@ -67,8 +79,6 @@ struct _ShellGlobal {
   const char *userdatadir;
   StFocusManager *focus_manager;
 
-  GdkWindow    *stage_window;
-
   guint work_count;
   GSList *leisure_closures;
   guint leisure_function_id;
@@ -82,9 +92,11 @@ struct _ShellGlobal {
 enum {
   PROP_0,
 
+  PROP_SESSION_TYPE,
   PROP_OVERLAY_GROUP,
   PROP_SCREEN,
   PROP_GDK_SCREEN,
+  PROP_DISPLAY,
   PROP_SCREEN_WIDTH,
   PROP_SCREEN_HEIGHT,
   PROP_STAGE,
@@ -126,7 +138,9 @@ shell_global_set_property(GObject         *object,
     case PROP_STAGE_INPUT_MODE:
       shell_global_set_stage_input_mode (global, g_value_get_enum (value));
       break;
-
+    case PROP_SESSION_TYPE:
+      global->session_type = g_value_get_enum (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -143,20 +157,26 @@ shell_global_get_property(GObject         *object,
 
   switch (prop_id)
     {
+    case PROP_SESSION_TYPE:
+      g_value_set_enum (value, shell_global_get_session_type (global));
+      break;
     case PROP_OVERLAY_GROUP:
-      g_value_set_object (value, meta_plugin_get_overlay_group (global->plugin));
+      g_value_set_object (value, meta_get_overlay_group_for_screen (global->meta_screen));
       break;
     case PROP_SCREEN:
-      g_value_set_object (value, shell_global_get_screen (global));
+      g_value_set_object (value, global->meta_screen);
       break;
     case PROP_GDK_SCREEN:
-      g_value_set_object (value, shell_global_get_gdk_screen (global));
+      g_value_set_object (value, global->gdk_screen);
+      break;
+    case PROP_DISPLAY:
+      g_value_set_object (value, global->meta_display);
       break;
     case PROP_SCREEN_WIDTH:
       {
         int width, height;
 
-        meta_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_screen_get_size (global->meta_screen, &width, &height);
         g_value_set_int (value, width);
       }
       break;
@@ -164,21 +184,21 @@ shell_global_get_property(GObject         *object,
       {
         int width, height;
 
-        meta_plugin_query_screen_size (global->plugin, &width, &height);
+        meta_screen_get_size (global->meta_screen, &width, &height);
         g_value_set_int (value, height);
       }
       break;
     case PROP_STAGE:
-      g_value_set_object (value, meta_plugin_get_stage (global->plugin));
+      g_value_set_object (value, global->stage);
       break;
     case PROP_STAGE_INPUT_MODE:
       g_value_set_enum (value, global->input_mode);
       break;
     case PROP_WINDOW_GROUP:
-      g_value_set_object (value, meta_plugin_get_window_group (global->plugin));
+      g_value_set_object (value, meta_get_window_group_for_screen (global->meta_screen));
       break;
     case PROP_BACKGROUND_ACTOR:
-      g_value_set_object (value, meta_plugin_get_background_actor (global->plugin));
+      g_value_set_object (value, meta_get_background_actor_for_screen (global->meta_screen));
       break;
     case PROP_WINDOW_MANAGER:
       g_value_set_object (value, global->wm);
@@ -238,8 +258,6 @@ shell_global_init (ShellGlobal *global)
   g_signal_connect (global->grab_notifier, "grab-notify", G_CALLBACK (grab_notify), global);
   global->gtk_grab_active = FALSE;
 
-  global->stage_window = NULL;
-
   global->input_mode = SHELL_STAGE_INPUT_MODE_NORMAL;
 
   ca_context_create (&global->sound_context);
@@ -320,6 +338,14 @@ shell_global_class_init (ShellGlobalClass *klass)
                     G_TYPE_STRING,
                     G_TYPE_STRING);
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_SESSION_TYPE,
+                                   g_param_spec_enum ("session-type",
+                                                      "Session Type",
+                                                      "The type of session",
+                                                      SHELL_TYPE_SESSION_TYPE,
+                                                      SHELL_SESSION_USER,
+                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class,
                                    PROP_OVERLAY_GROUP,
                                    g_param_spec_object ("overlay-group",
@@ -358,6 +384,14 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                      "Screen height, in pixels",
                                                      0, G_MAXINT, 1,
                                                      G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_DISPLAY,
+                                   g_param_spec_object ("display",
+                                                        "Display",
+                                                        "Metacity display object for the shell",
+                                                        META_TYPE_DISPLAY,
+                                                        G_PARAM_READABLE));
+
   g_object_class_install_property (gobject_class,
                                    PROP_STAGE,
                                    g_param_spec_object ("stage",
@@ -431,6 +465,37 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         G_PARAM_READABLE));
 }
 
+/**•
+ * _shell_global_init: (skip)•
+ * @first_property_name: the name of the first property
+ * @...: the value of the first property, followed optionally by more
+ *  name/value pairs, followed by %NULL
+ *•
+ * Initializes the shell global singleton with the construction-time
+ * properties.
+ *
+ * There are currently no such properties, so @first_property_name should
+ * always be %NULL.
+ *
+ * This call must be called before shell_global_get() and shouldn't be called
+ * more than once.
+ */
+void
+_shell_global_init (const char *first_property_name,
+                    ...)
+{
+  va_list argument_list;
+
+  g_return_if_fail (the_object == NULL);
+
+  va_start (argument_list, first_property_name);
+  the_object = SHELL_GLOBAL (g_object_new_valist (SHELL_TYPE_GLOBAL,
+                                                  first_property_name,
+                                                  argument_list));
+  va_end (argument_list);
+
+}
+
 /**
  * shell_global_get:
  *
@@ -441,9 +506,6 @@ shell_global_class_init (ShellGlobalClass *klass)
 ShellGlobal *
 shell_global_get (void)
 {
-  if (!the_object)
-    the_object = g_object_new (SHELL_TYPE_GLOBAL, 0);
-
   return the_object;
 }
 
@@ -462,14 +524,8 @@ focus_window_changed (MetaDisplay *display,
 static void
 shell_global_focus_stage (ShellGlobal *global)
 {
-  Display *xdpy;
-  ClutterActor *stage;
-  Window xstage;
-
-  stage = meta_plugin_get_stage (global->plugin);
-  xstage = clutter_x11_get_stage_window (CLUTTER_STAGE (stage));
-  xdpy = meta_plugin_get_xdisplay (global->plugin);
-  XSetInputFocus (xdpy, xstage, RevertToPointerRoot,
+  XSetInputFocus (global->xdisplay, global->stage_xwindow,
+                  RevertToPointerRoot,
                   shell_global_get_current_time (global));
 }
 
@@ -556,7 +612,7 @@ shell_global_set_cursor (ShellGlobal *global,
       g_return_if_reached ();
     }
 
-  cursor = gdk_cursor_new_from_name (gdk_display_get_default (), name);
+  cursor = gdk_cursor_new_from_name (global->gdk_display, name);
   if (!cursor)
     {
       GdkCursorType cursor_type;
@@ -581,15 +637,8 @@ shell_global_set_cursor (ShellGlobal *global,
         }
       cursor = gdk_cursor_new (cursor_type);
     }
-  if (!global->stage_window)
-    {
-      ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
 
-      global->stage_window = gdk_x11_window_foreign_new_for_display (gdk_display_get_default (),
-                                                                     clutter_x11_get_stage_window (stage));
-    }
-
-  gdk_window_set_cursor (global->stage_window, cursor);
+  gdk_window_set_cursor (global->stage_gdk_window, cursor);
 
   gdk_cursor_unref (cursor);
 }
@@ -603,10 +652,7 @@ shell_global_set_cursor (ShellGlobal *global,
 void
 shell_global_unset_cursor (ShellGlobal  *global)
 {
-  if (!global->stage_window) /* cursor has never been set */
-    return;
-
-  gdk_window_set_cursor (global->stage_window, NULL);
+  gdk_window_set_cursor (global->stage_gdk_window, NULL);
 }
 
 /**
@@ -623,9 +669,6 @@ void
 shell_global_set_stage_input_region (ShellGlobal *global,
                                      GSList      *rectangles)
 {
-  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
-  MetaDisplay *display = meta_screen_get_display (screen);
-  Display *xdpy = meta_display_get_xdisplay (display);
   MetaRectangle *rect;
   XRectangle *rects;
   int nrects, i;
@@ -645,9 +688,9 @@ shell_global_set_stage_input_region (ShellGlobal *global,
     }
 
   if (global->input_region)
-    XFixesDestroyRegion (xdpy, global->input_region);
+    XFixesDestroyRegion (global->xdisplay, global->input_region);
 
-  global->input_region = XFixesCreateRegion (xdpy, rects, nrects);
+  global->input_region = XFixesCreateRegion (global->xdisplay, rects, nrects);
   g_free (rects);
 
   /* set_stage_input_mode() will figure out whether or not we
@@ -664,7 +707,7 @@ shell_global_set_stage_input_region (ShellGlobal *global,
 MetaScreen *
 shell_global_get_screen (ShellGlobal  *global)
 {
-  return meta_plugin_get_screen (global->plugin);
+  return global->meta_screen;
 }
 
 /**
@@ -677,7 +720,18 @@ shell_global_get_gdk_screen (ShellGlobal *global)
 {
   g_return_val_if_fail (SHELL_IS_GLOBAL (global), NULL);
 
-  return gdk_screen_get_default ();
+  return global->gdk_screen;
+}
+
+/**
+ * shell_global_get_display:
+ *
+ * Return value: (transfer none): The default #MetaDisplay
+ */
+MetaDisplay *
+shell_global_get_display (ShellGlobal  *global)
+{
+  return global->meta_display;
 }
 
 /**
@@ -692,7 +746,7 @@ shell_global_get_window_actors (ShellGlobal *global)
 {
   g_return_val_if_fail (SHELL_IS_GLOBAL (global), NULL);
 
-  return meta_plugin_get_window_actors (global->plugin);
+  return meta_get_window_actors (global->meta_screen);
 }
 
 static void
@@ -735,26 +789,33 @@ void
 _shell_global_set_plugin (ShellGlobal *global,
                           MetaPlugin  *plugin)
 {
-  ClutterActor *stage;
-  MetaScreen *screen;
-  MetaDisplay *display;
-
   g_return_if_fail (SHELL_IS_GLOBAL (global));
   g_return_if_fail (global->plugin == NULL);
 
   global->plugin = plugin;
   global->wm = shell_wm_new (plugin);
 
-  stage = meta_plugin_get_stage (plugin);
+  global->meta_screen = meta_plugin_get_screen (plugin);
+  global->meta_display = meta_screen_get_display (global->meta_screen);
+  global->xdisplay = meta_display_get_xdisplay (global->meta_display);
 
-  g_signal_connect (stage, "notify::width",
+  global->gdk_display = gdk_x11_lookup_xdisplay (global->xdisplay);
+  global->gdk_screen = gdk_display_get_screen (global->gdk_display,
+                                               meta_screen_get_screen_number (global->meta_screen));
+
+  global->stage = CLUTTER_STAGE (meta_plugin_get_stage (plugin));
+  global->stage_xwindow = clutter_x11_get_stage_window (global->stage);
+  global->stage_gdk_window = gdk_x11_window_foreign_new_for_display (global->gdk_display,
+                                                                     global->stage_xwindow);
+
+  g_signal_connect (global->stage, "notify::width",
                     G_CALLBACK (global_stage_notify_width), global);
-  g_signal_connect (stage, "notify::height",
+  g_signal_connect (global->stage, "notify::height",
                     G_CALLBACK (global_stage_notify_height), global);
 
-  g_signal_connect (stage, "paint",
+  g_signal_connect (global->stage, "paint",
                     G_CALLBACK (global_stage_before_paint), global);
-  g_signal_connect_after (stage, "paint",
+  g_signal_connect_after (global->stage, "paint",
                           G_CALLBACK (global_stage_after_paint), global);
 
   shell_perf_log_define_event (shell_perf_log_get_default(),
@@ -766,12 +827,10 @@ _shell_global_set_plugin (ShellGlobal *global,
                                "End of stage page repaint",
                                "");
 
-  screen = meta_plugin_get_screen (global->plugin);
-  display = meta_screen_get_display (screen);
-  g_signal_connect (display, "notify::focus-window",
+  g_signal_connect (global->meta_display, "notify::focus-window",
                     G_CALLBACK (focus_window_changed), global);
 
-  global->focus_manager = st_focus_manager_get_for_stage (CLUTTER_STAGE (stage));
+  global->focus_manager = st_focus_manager_get_for_stage (global->stage);
 }
 
 GjsContext *
@@ -800,10 +859,8 @@ gboolean
 shell_global_begin_modal (ShellGlobal *global,
                           guint32      timestamp)
 {
-  ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
-  Window stagewin = clutter_x11_get_stage_window (stage);
-
-  return meta_plugin_begin_modal (global->plugin, stagewin, None, 0, timestamp);
+  return meta_plugin_begin_modal (global->plugin, global->stage_xwindow,
+                                  None, 0, timestamp);
 }
 
 /**
@@ -838,12 +895,9 @@ shell_global_create_pointer_barrier (ShellGlobal *global,
                                      int directions)
 {
 #if HAVE_XFIXESCREATEPOINTERBARRIER
-  Display *xdpy;
-
-  xdpy = meta_plugin_get_xdisplay (global->plugin);
-
   return (guint32)
-    XFixesCreatePointerBarrier (xdpy, DefaultRootWindow(xdpy),
+    XFixesCreatePointerBarrier (global->xdisplay,
+                                DefaultRootWindow (global->xdisplay),
                                 x1, y1,
                                 x2, y2,
                                 directions,
@@ -864,12 +918,9 @@ void
 shell_global_destroy_pointer_barrier (ShellGlobal *global, guint32 barrier)
 {
 #if HAVE_XFIXESCREATEPOINTERBARRIER
-  Display *xdpy;
-
   g_return_if_fail (barrier > 0);
 
-  xdpy = meta_plugin_get_xdisplay (global->plugin);
-  XFixesDestroyPointerBarrier (xdpy, (PointerBarrier)barrier);
+  XFixesDestroyPointerBarrier (global->xdisplay, (PointerBarrier)barrier);
 #endif
 }
 
@@ -1168,132 +1219,24 @@ grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
  */
 void shell_global_init_xdnd (ShellGlobal *global)
 {
+  Window output_window = meta_get_overlay_window (global->meta_screen);
   long xdnd_version = 5;
 
-  MetaScreen *screen = shell_global_get_screen (global);
-  Window output_window = meta_get_overlay_window (screen);
+  XChangeProperty (global->xdisplay, global->stage_xwindow,
+                   gdk_x11_get_xatom_by_name ("XdndAware"), XA_ATOM,
+                   32, PropModeReplace, (const unsigned char *)&xdnd_version, 1);
 
-  MetaDisplay *display = meta_screen_get_display (screen);
-  Display *xdisplay = meta_display_get_xdisplay (display);
-
-  ClutterStage *stage = CLUTTER_STAGE(meta_plugin_get_stage (global->plugin));
-  Window stage_win = clutter_x11_get_stage_window (stage);
-
-  XChangeProperty (xdisplay, stage_win, gdk_x11_get_xatom_by_name ("XdndAware"), XA_ATOM,
-                  32, PropModeReplace, (const unsigned char *)&xdnd_version, 1);
-
-  XChangeProperty (xdisplay, output_window, gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
-                  32, PropModeReplace, (const unsigned char *)&stage_win, 1);
+  XChangeProperty (global->xdisplay, output_window,
+                   gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
+                   32, PropModeReplace, (const unsigned char *)&global->stage_xwindow, 1);
 
   /*
    * XdndProxy is additionally set on the proxy window as verification that the
    * XdndProxy property on the target window isn't a left-over
    */
-  XChangeProperty (xdisplay, stage_win, gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
-                  32, PropModeReplace, (const unsigned char *)&stage_win, 1);
-}
-
-/**
- * shell_global_get_monitors:
- * @global: the #ShellGlobal
- *
- * Gets a list of the bounding boxes of the active screen's monitors.
- *
- * Return value: (transfer full) (element-type Meta.Rectangle): a list
- * of monitor bounding boxes.
- */
-GSList *
-shell_global_get_monitors (ShellGlobal *global)
-{
-  MetaScreen *screen = shell_global_get_screen (global);
-  GSList *monitors = NULL;
-  MetaRectangle rect;
-  int i;
-
-  for (i = meta_screen_get_n_monitors (screen) - 1; i >= 0; i--)
-    {
-      meta_screen_get_monitor_geometry (screen, i, &rect);
-      monitors = g_slist_prepend (monitors,
-                                  meta_rectangle_copy (&rect));
-    }
-  return monitors;
-}
-
-/**
- * shell_global_get_primary_monitor:
- * @global: the #ShellGlobal
- *
- * Gets the bounding box of the primary monitor (the one that the
- * panel is on).
- *
- * Return value: the bounding box of the primary monitor
- */
-MetaRectangle *
-shell_global_get_primary_monitor (ShellGlobal  *global)
-{
-  MetaScreen *screen = shell_global_get_screen (global);
-  MetaRectangle rect;
-  gint primary = 0;
-
-  primary = meta_screen_get_primary_monitor (screen);
-  meta_screen_get_monitor_geometry (screen, primary, &rect);
-
-  return meta_rectangle_copy (&rect);
-}
-
-/**
- * shell_global_get_primary_monitor_index:
- * @global: the #ShellGlobal
- *
- * Gets the index of the primary monitor (the one that the
- * panel is on).
- *
- * Return value: the index of the primary monitor
- */
-int
-shell_global_get_primary_monitor_index (ShellGlobal  *global)
-{
-  MetaScreen *screen = shell_global_get_screen (global);
-
-  return meta_screen_get_primary_monitor (screen);
-}
-
-/**
- * shell_global_get_focus_monitor:
- * @global: the #ShellGlobal
- *
- * Gets the bounding box of the monitor containing the window that
- * currently contains the keyboard focus.
- *
- * Return value: the bounding box of the focus monitor
- */
-MetaRectangle *
-shell_global_get_focus_monitor (ShellGlobal  *global)
-{
-  MetaScreen *screen = shell_global_get_screen (global);
-  MetaDisplay *display = meta_screen_get_display (screen);
-  MetaWindow *focus = meta_display_get_focus_window (display);
-  MetaRectangle rect, wrect;
-  int nmonitors, i;
-
-  if (focus)
-    {
-      meta_window_get_outer_rect (focus, &wrect);
-      nmonitors = meta_screen_get_n_monitors (screen);
-
-      /* Find the monitor that the top-left corner of @focus is on. */
-      for (i = 0; i < nmonitors; i++)
-        {
-          meta_screen_get_monitor_geometry (screen, i, &rect);
-
-          if (rect.x <= wrect.x && rect.y <= wrect.y &&
-              rect.x + rect.width > wrect.x &&
-              rect.y + rect.height > wrect.y)
-            return meta_rectangle_copy (&rect);
-        }
-    }
-
-  return shell_global_get_primary_monitor (global);
+  XChangeProperty (global->xdisplay, global->stage_xwindow,
+                   gdk_x11_get_xatom_by_name ("XdndProxy"), XA_WINDOW,
+                   32, PropModeReplace, (const unsigned char *)&global->stage_xwindow, 1);
 }
 
 /**
@@ -1316,7 +1259,7 @@ shell_global_get_pointer (ShellGlobal         *global,
 {
   GdkModifierType raw_mods;
 
-  gdk_display_get_pointer (gdk_display_get_default (), NULL, x, y, &raw_mods);
+  gdk_display_get_pointer (global->gdk_display, NULL, x, y, &raw_mods);
   *mods = raw_mods & GDK_MODIFIER_MASK;
 }
 
@@ -1335,7 +1278,7 @@ shell_global_sync_pointer (ShellGlobal *global)
   GdkModifierType mods;
   ClutterMotionEvent event;
 
-  gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, &mods);
+  gdk_display_get_pointer (global->gdk_display, NULL, &x, &y, &mods);
 
   event.type = CLUTTER_MOTION;
   event.time = shell_global_get_current_time (global);
@@ -1344,7 +1287,7 @@ shell_global_sync_pointer (ShellGlobal *global)
    * pointer is not inside the bounds of the stage given the current
    * stage_input_mode. For our current purposes however, this works.
    */
-  event.stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
+  event.stage = global->stage;
   event.x = x;
   event.y = y;
   event.modifier_state = mods;
@@ -1389,7 +1332,6 @@ guint32
 shell_global_get_current_time (ShellGlobal *global)
 {
   guint32 time;
-  MetaDisplay *display;
   const ClutterEvent *clutter_event;
 
   /* In case we have a xdnd timestamp use it */
@@ -1409,8 +1351,7 @@ shell_global_get_current_time (ShellGlobal *global)
      to clutter_get_current_event_time().
    */
 
-  display = meta_screen_get_display (shell_global_get_screen (global));
-  time = meta_display_get_current_time (display);
+  time = meta_display_get_current_time (global->meta_display);
   if (time != CLUTTER_CURRENT_TIME)
       return time;
   /*
@@ -1444,7 +1385,7 @@ shell_global_create_app_launch_context (ShellGlobal *global)
 
   // Make sure that the app is opened on the current workspace even if
   // the user switches before it starts
-  gdk_app_launch_context_set_desktop (context, meta_screen_get_active_workspace_index (shell_global_get_screen (global)));
+  gdk_app_launch_context_set_desktop (context, meta_screen_get_active_workspace_index (global->meta_screen));
 
   return (GAppLaunchContext *)context;
 }
@@ -1632,15 +1573,9 @@ shell_global_cancel_theme_sound (ShellGlobal *global,
 gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
                                          XEvent       *xev)
 {
-  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
-  Window output_window = meta_get_overlay_window (screen);
-  MetaDisplay *display = meta_screen_get_display (screen);
-  Display *xdisplay = meta_display_get_xdisplay (display);
+  Window output_window = meta_get_overlay_window (global->meta_screen);
 
-  ClutterStage *stage = CLUTTER_STAGE (meta_plugin_get_stage (global->plugin));
-  Window stage_win = clutter_x11_get_stage_window (stage);
-
-  if (xev->xany.window != output_window && xev->xany.window != stage_win)
+  if (xev->xany.window != output_window && xev->xany.window != global->stage_xwindow)
     return FALSE;
 
   if (xev->xany.type == ClientMessage && xev->xclient.message_type == gdk_x11_get_xatom_by_name ("XdndPosition"))
@@ -1650,7 +1585,7 @@ gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
 
       memset (&xevent, 0, sizeof(xevent));
       xevent.xany.type = ClientMessage;
-      xevent.xany.display = xdisplay;
+      xevent.xany.display = global->xdisplay;
       xevent.xclient.window = src;
       xevent.xclient.message_type = gdk_x11_get_xatom_by_name ("XdndStatus");
       xevent.xclient.format = 32;
@@ -1659,7 +1594,7 @@ gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
       xevent.xclient.data.l[1] = 2;
       xevent.xclient.data.l[4] = None;
 
-      XSendEvent (xdisplay, src, False, 0, &xevent);
+      XSendEvent (global->xdisplay, src, False, 0, &xevent);
 
       /* Store the timestamp of the xdnd position event */
       global->xdnd_timestamp = xev->xclient.data.l[3];
@@ -1738,3 +1673,261 @@ shell_global_launch_calendar_server (ShellGlobal *global)
 
   g_free (calendar_server_exe);
 }
+
+static void
+grab_screenshot (ClutterActor *stage,
+                 _screenshot_data *screenshot_data)
+{
+  MetaScreen *screen = shell_global_get_screen (screenshot_data->global);
+  cairo_status_t status;
+  cairo_surface_t *image;
+  guchar *data;
+  int width, height;
+
+  meta_plugin_query_screen_size (screenshot_data->global->plugin, &width, &height);
+  image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
+  data = cairo_image_surface_get_data (image);
+
+  cogl_flush();
+
+  cogl_read_pixels (0, 0, width, height, COGL_READ_PIXELS_COLOR_BUFFER, CLUTTER_CAIRO_FORMAT_ARGB32, data);
+
+  cairo_surface_mark_dirty (image);
+
+  if (meta_screen_get_n_monitors (screen) > 1)
+    {
+      cairo_region_t *screen_region = cairo_region_create ();
+      cairo_region_t *stage_region;
+      MetaRectangle monitor_rect;
+      cairo_rectangle_int_t stage_rect;
+      int i;
+      cairo_t *cr;
+
+      for (i = meta_screen_get_n_monitors (screen) - 1; i >= 0; i--)
+        {
+          meta_screen_get_monitor_geometry (screen, i, &monitor_rect);
+          cairo_region_union_rectangle (screen_region, (const cairo_rectangle_int_t *) &monitor_rect);
+        }
+
+      stage_rect.x = 0;
+      stage_rect.y = 0;
+      stage_rect.width = width;
+      stage_rect.height = height;
+
+      stage_region = cairo_region_create_rectangle ((const cairo_rectangle_int_t *) &stage_rect);
+      cairo_region_xor (stage_region, screen_region);
+      cairo_region_destroy (screen_region);
+
+      cr = cairo_create (image);
+
+      for (i = 0; i < cairo_region_num_rectangles (stage_region); i++)
+        {
+          cairo_rectangle_int_t rect;
+          cairo_region_get_rectangle (stage_region, i, &rect);
+          cairo_rectangle (cr, (double) rect.x, (double) rect.y, (double) rect.width, (double) rect.height);
+          cairo_fill (cr);
+        }
+
+      cairo_destroy (cr);
+      cairo_region_destroy (stage_region);
+    }
+
+
+  status = cairo_surface_write_to_png (image, screenshot_data->filename);
+  cairo_surface_destroy (image);
+
+  if (screenshot_data->callback)
+    screenshot_data->callback (screenshot_data->global, status == CAIRO_STATUS_SUCCESS);
+
+  g_signal_handlers_disconnect_by_func (stage, (void *)grab_screenshot, (gpointer)screenshot_data);
+  g_free (screenshot_data->filename);
+  g_free (screenshot_data);
+}
+
+static void
+grab_area_screenshot (ClutterActor *stage,
+                      _screenshot_data *screenshot_data)
+{
+  cairo_status_t status;
+  cairo_surface_t *image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, screenshot_data->width, screenshot_data->height);
+  guchar *data = cairo_image_surface_get_data (image);
+
+  cogl_flush();
+
+  cogl_read_pixels (screenshot_data->x, screenshot_data->y, screenshot_data->width, screenshot_data->height,
+                    COGL_READ_PIXELS_COLOR_BUFFER, CLUTTER_CAIRO_FORMAT_ARGB32, data);
+
+  cairo_surface_mark_dirty (image);
+  status = cairo_surface_write_to_png (image, screenshot_data->filename);
+  cairo_surface_destroy (image);
+
+  if (screenshot_data->callback)
+    screenshot_data->callback (screenshot_data->global, status == CAIRO_STATUS_SUCCESS);
+
+  g_signal_handlers_disconnect_by_func (stage, (void *)grab_area_screenshot, (gpointer)screenshot_data);
+  g_free (screenshot_data->filename);
+  g_free (screenshot_data);
+}
+
+/**
+ * shell_global_screenshot:
+ * @global: the #ShellGlobal
+ * @filename: The filename for the screenshot
+ * @callback: (scope async): function to call returning success or failure
+ * of the async grabbing
+ *
+ * Takes a screenshot of the whole screen
+ * in @filename as png image.
+ *
+ */
+void
+shell_global_screenshot (ShellGlobal  *global,
+                        const char *filename,
+                        ShellGlobalScreenshotCallback callback)
+{
+  ClutterActor *stage;
+  _screenshot_data *data = g_new0 (_screenshot_data, 1);
+
+  data->global = global;
+  data->filename = g_strdup (filename);
+  data->callback = callback;
+
+  stage = CLUTTER_ACTOR (meta_plugin_get_stage (global->plugin));
+
+  g_signal_connect_after (stage, "paint", G_CALLBACK (grab_screenshot), (gpointer)data);
+
+  clutter_actor_queue_redraw (stage);
+}
+
+/**
+ * shell_global_screenshot_area:
+ * @global: the #ShellGlobal
+ * @x: The X coordinate of the area
+ * @y: The Y coordinate of the area
+ * @width: The width of the area
+ * @height: The height of the area
+ * @filename: The filename for the screenshot
+ * @callback: (scope async): function to call returning success or failure
+ * of the async grabbing
+ *
+ * Takes a screenshot of the passed in area and saves it
+ * in @filename as png image.
+ *
+ */
+void
+shell_global_screenshot_area (ShellGlobal  *global,
+                              int x,
+                              int y,
+                              int width,
+                              int height,
+                              const char *filename,
+                              ShellGlobalScreenshotCallback callback)
+{
+  ClutterActor *stage;
+  _screenshot_data *data = g_new0 (_screenshot_data, 1);
+
+  data->global = global;
+  data->filename = g_strdup (filename);
+  data->x = x;
+  data->y = y;
+  data->width = width;
+  data->height = height;
+  data->callback = callback;
+
+  stage = CLUTTER_ACTOR (meta_plugin_get_stage (global->plugin));
+
+  g_signal_connect_after (stage, "paint", G_CALLBACK (grab_area_screenshot), (gpointer)data);
+
+  clutter_actor_queue_redraw (stage);
+}
+
+/**
+ * shell_global_screenshot_window:
+ * @global: the #ShellGlobal
+ * @include_frame: Whether to include the frame or not
+ *
+ * @filename: The filename for the screenshot
+ *
+ * Takes a screenshot of the focused window (optionally omitting the frame)
+ * in @filename as png image.
+ *
+ * Return value: success or failure.
+ */
+gboolean
+shell_global_screenshot_window (ShellGlobal  *global,
+                                gboolean include_frame,
+                                const char *filename)
+{
+  CoglHandle texture;
+  cairo_surface_t *image;
+  guchar *data;
+
+  MetaScreen *screen = meta_plugin_get_screen (global->plugin);
+  MetaDisplay *display = meta_screen_get_display (screen);
+  MetaWindow *window = meta_display_get_focus_window (display);
+  ClutterActor *window_actor;
+
+  cairo_status_t status;
+
+  window_actor = CLUTTER_ACTOR (meta_window_get_compositor_private (window));
+  texture = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (meta_window_actor_get_texture (META_WINDOW_ACTOR (window_actor))));
+
+  if (!include_frame)
+    {
+      MetaRectangle *window_rect = meta_window_get_rect (window);
+      texture = cogl_texture_new_from_sub_texture (texture,
+                                                   window_rect->x,
+                                                   window_rect->y,
+                                                   window_rect->width,
+                                                   window_rect->height);
+
+      image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+                                          window_rect->width,
+                                          window_rect->height);
+    }
+  else
+    image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+                                        clutter_actor_get_width (window_actor),
+                                        clutter_actor_get_height (window_actor));
+
+  data = cairo_image_surface_get_data (image);
+
+  cogl_flush();
+
+  cogl_texture_get_data (texture, CLUTTER_CAIRO_FORMAT_ARGB32, 0, data);
+
+  cairo_surface_mark_dirty (image);
+  status = cairo_surface_write_to_png (image, filename);
+  cairo_surface_destroy (image);
+
+  return status == CAIRO_STATUS_SUCCESS;
+}
+
+/**
+ * shell_global_get_session_type:
+ * @global: The #ShellGlobal.
+ *
+ * Gets the type of session gnome-shell provides.
+ *
+ * The type determines what UI elements are displayed,
+ * what keybindings work, and generally how the shell
+ * behaves.
+ *
+ * A session type of #SHELL_SESSION_USER means gnome-shell
+ * will enable the activities overview, status menu, run dialog,
+ * etc. This is the default.
+ *
+ * A session type of #SHELL_SESSION_GDM means gnome-shell
+ * will enable a login dialog and run in a more confined
+ * way. This type is suitable for the display manager.
+ *
+ * Returns the type of session gnome-shell is providing.
+ */
+ShellSessionType
+shell_global_get_session_type (ShellGlobal  *global)
+{
+  g_return_val_if_fail (SHELL_IS_GLOBAL (global),
+                        SHELL_SESSION_USER);
+
+  return global->session_type;
+}
