@@ -75,14 +75,6 @@ struct _StScrollBarPrivate
   ClutterAnimation *paging_animation;
 
   guint             vertical : 1;
-
-  /* We want to keep track of whether we have a currently valid
-   * allocation or not. This isn't exported from ClutterActor
-   * so we need to shadow the computations and track it ourselves.
-   *
-   * http://bugzilla.openedhand.com/show_bug.cgi?id=2024
-   */
-  guint             needs_allocation : 1;
 };
 
 enum
@@ -110,6 +102,8 @@ handle_button_press_event_cb (ClutterActor       *actor,
                               ClutterButtonEvent *event,
                               StScrollBar        *bar);
 
+static void stop_scrolling (StScrollBar *bar);
+
 static void
 st_scroll_bar_get_property (GObject    *gobject,
                             guint       prop_id,
@@ -271,6 +265,8 @@ st_scroll_bar_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->unmap (actor);
 
+  stop_scrolling (ST_SCROLL_BAR (actor));
+
   clutter_actor_unmap (priv->bw_stepper);
   clutter_actor_unmap (priv->fw_stepper);
   clutter_actor_unmap (priv->trough);
@@ -279,28 +275,6 @@ st_scroll_bar_unmap (ClutterActor *actor)
     clutter_actor_unmap (priv->handle);
 }
 
-static void
-st_scroll_bar_parent_set (ClutterActor  *actor,
-                          ClutterActor   *old_parent)
-{
-  StScrollBarPrivate *priv = ST_SCROLL_BAR (actor)->priv;
-
-  priv->needs_allocation = TRUE;
-
-  if (CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->parent_set)
-    CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->parent_set (actor, old_parent);
-}
-
-static void
-st_scroll_bar_queue_relayout (ClutterActor *actor)
-{
-  StScrollBarPrivate *priv = ST_SCROLL_BAR (actor)->priv;
-
-  priv->needs_allocation = TRUE;
-
-  CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->queue_relayout (actor);
-}
-
 static void
 scroll_bar_allocate_children (StScrollBar           *bar,
                               const ClutterActorBox *box,
@@ -577,9 +551,6 @@ st_scroll_bar_allocate (ClutterActor          *actor,
                         ClutterAllocationFlags flags)
 {
   StScrollBar *bar = ST_SCROLL_BAR (actor);
-  StScrollBarPrivate *priv = bar->priv;
-
-  priv->needs_allocation = FALSE;
 
   /* Chain up */
   CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->allocate (actor, box, flags);
@@ -590,7 +561,6 @@ st_scroll_bar_allocate (ClutterActor          *actor,
 static void
 scroll_bar_update_positions (StScrollBar *bar)
 {
-  StScrollBarPrivate *priv = bar->priv;
   ClutterActorBox box;
 
   /* Due to a change in the adjustments, we need to reposition our
@@ -604,7 +574,7 @@ scroll_bar_update_positions (StScrollBar *bar)
    * allocate our children if we already have a valid allocation, and
    * otherwise just wait for the queued relayout.
    */
-  if (priv->needs_allocation)
+  if (!clutter_actor_has_allocation (CLUTTER_ACTOR (bar)))
     return;
 
   clutter_actor_get_allocation_box (CLUTTER_ACTOR (bar), &box);
@@ -714,8 +684,6 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
 
   actor_class->get_preferred_width  = st_scroll_bar_get_preferred_width;
   actor_class->get_preferred_height = st_scroll_bar_get_preferred_height;
-  actor_class->parent_set     = st_scroll_bar_parent_set;
-  actor_class->queue_relayout = st_scroll_bar_queue_relayout;
   actor_class->allocate       = st_scroll_bar_allocate;
   actor_class->paint          = st_scroll_bar_paint;
   actor_class->pick           = st_scroll_bar_pick;
@@ -806,6 +774,22 @@ move_slider (StScrollBar *bar,
   st_adjustment_set_value (priv->adjustment, position);
 }
 
+static void
+stop_scrolling (StScrollBar *bar)
+{
+  ClutterActor *stage;
+
+  if (!bar->priv->capture_handler)
+    return;
+
+  stage = clutter_actor_get_stage (bar->priv->trough);
+  g_signal_handler_disconnect (stage, bar->priv->capture_handler);
+  bar->priv->capture_handler = 0;
+
+  clutter_set_motion_events_enabled (TRUE);
+  g_signal_emit (bar, signals[SCROLL_STOP], 0);
+}
+
 static gboolean
 handle_capture_event_cb (ClutterActor *trough,
                          ClutterEvent *event,
@@ -822,19 +806,11 @@ handle_capture_event_cb (ClutterActor *trough,
     {
       ClutterActor *stage, *target;
 
-      stage = clutter_actor_get_stage(bar->priv->trough);
-
-      if (bar->priv->capture_handler)
-        {
-          g_signal_handler_disconnect (stage, bar->priv->capture_handler);
-          bar->priv->capture_handler = 0;
-        }
-
-      clutter_set_motion_events_enabled (TRUE);
-      g_signal_emit (bar, signals[SCROLL_STOP], 0);
+      stop_scrolling (bar);
 
       /* check if the mouse pointer has left the handle during the drag and
        * remove the hover state if it has */
+      stage = clutter_actor_get_stage (bar->priv->trough);
       target = clutter_stage_get_actor_at_pos ((ClutterStage*) stage,
                                                CLUTTER_PICK_REACTIVE,
                                                ((ClutterButtonEvent*) event)->x,
@@ -843,8 +819,6 @@ handle_capture_event_cb (ClutterActor *trough,
         {
           st_widget_remove_style_pseudo_class ((StWidget*) bar->priv->handle, "hover");
         }
-
-
     }
 
   return TRUE;
@@ -1225,8 +1199,6 @@ st_scroll_bar_init (StScrollBar *self)
 
   g_signal_connect (self, "notify::reactive",
                     G_CALLBACK (st_scroll_bar_notify_reactive), NULL);
-
-  self->priv->needs_allocation = TRUE;
 }
 
 StWidget *
