@@ -143,7 +143,7 @@ st_texture_cache_init (StTextureCache *self)
                                                    g_free, cogl_handle_unref);
   self->priv->outstanding_requests = g_hash_table_new_full (g_str_hash, g_str_equal,
                                                             g_free, NULL);
-  self->priv->thumbnails = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
+  self->priv->thumbnails = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
 }
 
 static void
@@ -777,6 +777,7 @@ typedef struct {
   char *key;
   char *uri;
   gboolean thumbnail;
+  gboolean enforced_square;
   char *mimetype;
   GtkRecentInfo *recent_info;
   char *checksum;
@@ -788,15 +789,47 @@ typedef struct {
 } AsyncTextureLoadData;
 
 static CoglHandle
-pixbuf_to_cogl_handle (GdkPixbuf *pixbuf)
+pixbuf_to_cogl_handle (GdkPixbuf *pixbuf,
+                       gboolean   add_padding)
 {
-  return cogl_texture_new_from_data (gdk_pixbuf_get_width (pixbuf),
-                                     gdk_pixbuf_get_height (pixbuf),
-                                     COGL_TEXTURE_NONE,
-                                     gdk_pixbuf_get_has_alpha (pixbuf) ? COGL_PIXEL_FORMAT_RGBA_8888 : COGL_PIXEL_FORMAT_RGB_888,
-                                     COGL_PIXEL_FORMAT_ANY,
-                                     gdk_pixbuf_get_rowstride (pixbuf),
-                                     gdk_pixbuf_get_pixels (pixbuf));
+  CoglHandle texture, offscreen;
+  CoglColor clear_color;
+  int width, height;
+  guint size;
+
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
+  size = MAX (width, height);
+
+  if (!add_padding || width == height)
+    return cogl_texture_new_from_data (width,
+                                       height,
+                                       COGL_TEXTURE_NONE,
+                                       gdk_pixbuf_get_has_alpha (pixbuf) ? COGL_PIXEL_FORMAT_RGBA_8888 : COGL_PIXEL_FORMAT_RGB_888,
+                                       COGL_PIXEL_FORMAT_ANY,
+                                       gdk_pixbuf_get_rowstride (pixbuf),
+                                       gdk_pixbuf_get_pixels (pixbuf));
+
+  texture = cogl_texture_new_with_size (size, size,
+                                        COGL_TEXTURE_NO_SLICING,
+                                        COGL_PIXEL_FORMAT_ANY);
+
+  offscreen = cogl_offscreen_new_to_texture (texture);
+  cogl_color_set_from_4ub (&clear_color, 0, 0, 0, 0);
+  cogl_push_framebuffer (offscreen);
+  cogl_clear (&clear_color, COGL_BUFFER_BIT_COLOR);
+  cogl_pop_framebuffer ();
+  cogl_handle_unref (offscreen);
+
+  cogl_texture_set_region (texture,
+                           0, 0,
+                           (size - width) / 2, (size - height) / 2,
+                           width, height,
+                           width, height,
+                           gdk_pixbuf_get_has_alpha (pixbuf) ? COGL_PIXEL_FORMAT_RGBA_8888 : COGL_PIXEL_FORMAT_RGB_888,
+                           gdk_pixbuf_get_rowstride (pixbuf),
+                           gdk_pixbuf_get_pixels (pixbuf));
+  return texture;
 }
 
 static cairo_surface_t *
@@ -882,7 +915,7 @@ on_pixbuf_loaded (GObject      *source,
   if (pixbuf == NULL)
     goto out;
 
-  texdata = pixbuf_to_cogl_handle (pixbuf);
+  texdata = pixbuf_to_cogl_handle (pixbuf, data->enforced_square);
 
   g_object_unref (pixbuf);
 
@@ -956,7 +989,7 @@ st_texture_cache_reset_texture (StTextureCachePropertyBind *bind,
 
   if (pixbuf != NULL)
     {
-      texdata = pixbuf_to_cogl_handle (pixbuf);
+      texdata = pixbuf_to_cogl_handle (pixbuf, FALSE);
       g_object_unref (pixbuf);
 
       clutter_texture_set_cogl_texture (bind->texture, texdata);
@@ -1121,7 +1154,6 @@ create_texture_and_ensure_request (StTextureCache        *cache,
     {
       /* Not cached and no pending request, create it */
       *request = g_new0 (AsyncTextureLoadData, 1);
-      (*request)->key = g_strdup (key);
       g_hash_table_insert (cache->priv->outstanding_requests, g_strdup (key), *request);
     }
   else
@@ -1176,28 +1208,32 @@ load_gicon_with_colors (StTextureCache    *cache,
   info = gtk_icon_theme_lookup_by_gicon (theme, icon, size, GTK_ICON_LOOKUP_USE_BUILTIN);
   if (info != NULL)
     {
+      /* Transfer ownership of key */
+      request->key = key;
       /* hardcoded here for now; we should actually blow this away on
        * icon theme changes probably */
-      request->key = g_strdup (key);
       request->policy = ST_TEXTURE_CACHE_POLICY_FOREVER;
       request->icon = g_object_ref (icon);
       request->icon_info = info;
       request->width = request->height = size;
+      request->enforced_square = TRUE;
 
       load_icon_pixbuf_async (cache, icon, info, size, colors, NULL, on_pixbuf_loaded, request);
     }
   else
     {
       /* Blah; we failed to find the icon, but we've added our texture to the outstanding
-       * requests.  In that case, just undo what create_texture_lookup_status did.
+       * requests.  In that case, just undo what create_texture_and_ensure_request() did.
        */
        g_slist_foreach (request->textures, (GFunc) g_object_unref, NULL);
        g_slist_free (request->textures);
        g_free (request);
        g_hash_table_remove (cache->priv->outstanding_requests, key);
+       g_free (key);
+       g_object_unref (texture);
+       texture = NULL;
     }
 
-  g_free (key);
   return CLUTTER_ACTOR (texture);
 }
 
@@ -1216,7 +1252,7 @@ load_gicon_with_colors (StTextureCache    *cache,
  * This will load @icon as a full-color icon; if you want a symbolic
  * icon, you must use st_texture_cache_load_icon_name().
  *
- * Return Value: (transfer none): A new #ClutterActor for the icon
+ * Return Value: (transfer none): A new #ClutterActor for the icon, or %NULL if not found
  */
 ClutterActor *
 st_texture_cache_load_gicon (StTextureCache    *cache,
@@ -1245,7 +1281,7 @@ load_from_pixbuf (GdkPixbuf *pixbuf)
 
   clutter_actor_set_size (CLUTTER_ACTOR (texture), width, height);
 
-  texdata = pixbuf_to_cogl_handle (pixbuf);
+  texdata = pixbuf_to_cogl_handle (pixbuf, FALSE);
 
   set_texture_cogl_texture (texture, texdata);
 
@@ -1391,6 +1427,46 @@ st_texture_cache_load_sliced_image (StTextureCache    *cache,
  * icon you are loading, use %ST_ICON_FULLCOLOR.
  */
 
+/* generates names like g_themed_icon_new_with_default_fallbacks(),
+ * but *only* symbolic names
+ */
+static char **
+symbolic_names_for_icon (const char *name)
+{
+  char **parts, **names;
+  int i, numnames;
+
+  parts = g_strsplit (name, "-", -1);
+  numnames = g_strv_length (parts);
+  names = g_new (char *, numnames + 1);
+  for (i = 0; parts[i]; i++)
+    {
+      if (i == 0)
+        {
+          names[i] = g_strdup_printf ("%s-symbolic", parts[i]);
+        }
+      else
+        {
+          names[i] = g_strdup_printf ("%.*s-%s-symbolic",
+                                      (int) (strlen (names[i - 1]) - strlen ("-symbolic")),
+                                      names[i - 1], parts[i]);
+        }
+    }
+  names[i] = NULL;
+
+  g_strfreev (parts);
+
+  /* need to reverse here, because longest (most specific)
+     name has to come first */
+  for (i = 0; i < (numnames / 2); i++) {
+    char *tmp = names[i];
+    names[i] = names[numnames - i - 1];
+    names[numnames - i - 1] = tmp;
+  }
+
+  return names;
+}
+
 /**
  * st_texture_cache_load_icon_name:
  * @cache: The texture cache instance
@@ -1414,24 +1490,41 @@ st_texture_cache_load_icon_name (StTextureCache    *cache,
 {
   ClutterActor *texture;
   GIcon *themed;
-  char *symbolic;
+  char **names;
 
   g_return_val_if_fail (!(icon_type == ST_ICON_SYMBOLIC && theme_node == NULL), NULL);
 
   switch (icon_type)
     {
     case ST_ICON_APPLICATION:
+      themed = g_themed_icon_new (name);
+      texture = load_gicon_with_colors (cache, themed, size, NULL);
+      g_object_unref (themed);
+      if (texture == NULL)
+        {
+          themed = g_themed_icon_new ("application-x-executable");
+          texture = load_gicon_with_colors (cache, themed, size, NULL);
+          g_object_unref (themed);
+        }
+      return CLUTTER_ACTOR (texture);
+      break;
     case ST_ICON_DOCUMENT:
       themed = g_themed_icon_new (name);
       texture = load_gicon_with_colors (cache, themed, size, NULL);
       g_object_unref (themed);
+      if (texture == NULL)
+        {
+          themed = g_themed_icon_new ("x-office-document");
+          texture = load_gicon_with_colors (cache, themed, size, NULL);
+          g_object_unref (themed);
+        }
 
       return CLUTTER_ACTOR (texture);
       break;
     case ST_ICON_SYMBOLIC:
-      symbolic = g_strconcat (name, "-symbolic", NULL);
-      themed = g_themed_icon_new_with_default_fallbacks ((const gchar*)symbolic);
-      g_free (symbolic);
+      names = symbolic_names_for_icon (name);
+      themed = g_themed_icon_new_from_names (names, -1);
+      g_strfreev (names);
       texture = load_gicon_with_colors (cache, themed, size,
                                         st_theme_node_get_icon_colors (theme_node));
       g_object_unref (themed);
@@ -1442,6 +1535,12 @@ st_texture_cache_load_icon_name (StTextureCache    *cache,
       themed = g_themed_icon_new_with_default_fallbacks (name);
       texture = load_gicon_with_colors (cache, themed, size, NULL);
       g_object_unref (themed);
+      if (texture == NULL)
+        {
+          themed = g_themed_icon_new ("image-missing");
+          texture = load_gicon_with_colors (cache, themed, size, NULL);
+          g_object_unref (themed);
+        }
 
       return CLUTTER_ACTOR (texture);
       break;
@@ -1509,7 +1608,7 @@ st_texture_cache_load_uri_sync_to_cogl_texture (StTextureCache *cache,
       if (!pixbuf)
         goto out;
 
-      texdata = pixbuf_to_cogl_handle (pixbuf);
+      texdata = pixbuf_to_cogl_handle (pixbuf, FALSE);
       g_object_unref (pixbuf);
 
       if (policy == ST_TEXTURE_CACHE_POLICY_FOREVER)
@@ -1760,7 +1859,7 @@ st_texture_cache_load_from_data (StTextureCache    *cache,
           return NULL;
         }
 
-      texdata = pixbuf_to_cogl_handle (pixbuf);
+      texdata = pixbuf_to_cogl_handle (pixbuf, TRUE);
       g_object_unref (pixbuf);
 
       set_texture_cogl_texture (texture, texdata);
@@ -1884,6 +1983,7 @@ st_texture_cache_load_thumbnail (StTextureCache    *cache,
       data->thumbnail = TRUE;
       data->width = size;
       data->height = size;
+      data->enforced_square = TRUE;
       data->textures = g_slist_prepend (data->textures, g_object_ref (texture));
       load_thumbnail_async (cache, uri, mimetype, size, NULL, on_pixbuf_loaded, data);
     }
@@ -1960,6 +2060,7 @@ st_texture_cache_load_recent_thumbnail (StTextureCache    *cache,
       data->recent_info = gtk_recent_info_ref (info);
       data->width = size;
       data->height = size;
+      data->enforced_square = TRUE;
       data->textures = g_slist_prepend (data->textures, g_object_ref (texture));
       load_recent_thumbnail_async (cache, info, size, NULL, on_pixbuf_loaded, data);
     }
