@@ -167,11 +167,7 @@ create_corner_material (StCornerSpec *corner)
 
   texture = cogl_texture_new_from_data (size, size,
                                         COGL_TEXTURE_NONE,
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-                                        COGL_PIXEL_FORMAT_BGRA_8888_PRE,
-#else
-                                        COGL_PIXEL_FORMAT_ARGB_8888_PRE,
-#endif
+                                        CLUTTER_CAIRO_FORMAT_ARGB32,
                                         COGL_PIXEL_FORMAT_ANY,
                                         rowstride,
                                         data);
@@ -257,6 +253,57 @@ over (const ClutterColor *source,
   unpremultiply (result);
 }
 
+/*
+ * st_theme_node_reduce_border_radius:
+ * @node: a #StThemeNode
+ * @corners: (array length=4) (out): reduced corners
+ *
+ * Implements the corner overlap algorithm mentioned at
+ * http://www.w3.org/TR/css3-background/#corner-overlap
+ */
+static void
+st_theme_node_reduce_border_radius (StThemeNode  *node,
+                                    guint        *corners)
+{
+  gfloat scale;
+  guint sum;
+
+  scale = 1.0;
+
+  /* top */
+  sum = node->border_radius[ST_CORNER_TOPLEFT]
+    + node->border_radius[ST_CORNER_TOPRIGHT];
+
+  if (sum > 0)
+    scale = MIN (node->alloc_width / sum, scale);
+
+  /* right */
+  sum = node->border_radius[ST_CORNER_TOPRIGHT]
+    + node->border_radius[ST_CORNER_BOTTOMRIGHT];
+
+  if (sum > 0)
+    scale = MIN (node->alloc_height / sum, scale);
+
+  /* bottom */
+  sum = node->border_radius[ST_CORNER_BOTTOMLEFT]
+    + node->border_radius[ST_CORNER_BOTTOMRIGHT];
+
+  if (sum > 0)
+    scale = MIN (node->alloc_width / sum, scale);
+
+  /* left */
+  sum = node->border_radius[ST_CORNER_BOTTOMLEFT]
+    + node->border_radius[ST_CORNER_TOPLEFT];
+
+  if (sum > 0)
+    scale = MIN (node->alloc_height / sum, scale);
+
+  corners[ST_CORNER_TOPLEFT]     = node->border_radius[ST_CORNER_TOPLEFT]     * scale;
+  corners[ST_CORNER_TOPRIGHT]    = node->border_radius[ST_CORNER_TOPRIGHT]    * scale;
+  corners[ST_CORNER_BOTTOMLEFT]  = node->border_radius[ST_CORNER_BOTTOMLEFT]  * scale;
+  corners[ST_CORNER_BOTTOMRIGHT] = node->border_radius[ST_CORNER_BOTTOMRIGHT] * scale;
+}
+
 static void
 st_theme_node_get_corner_border_widths (StThemeNode *node,
                                         StCorner     corner_id,
@@ -301,13 +348,15 @@ st_theme_node_lookup_corner (StThemeNode    *node,
   StTextureCache *cache;
   StCornerSpec corner;
   LoadCornerData data;
+  guint radius[4];
 
   if (node->border_radius[corner_id] == 0)
     return COGL_INVALID_HANDLE;
 
   cache = st_texture_cache_get_default ();
 
-  corner.radius = node->border_radius[corner_id];
+  st_theme_node_reduce_border_radius (node, radius);
+  corner.radius = radius[corner_id];
   corner.color = node->background_color;
   st_theme_node_get_corner_border_widths (node, corner_id,
                                           &corner.border_width_1,
@@ -602,6 +651,89 @@ create_cairo_pattern_of_background_image (StThemeNode *node,
   return pattern;
 }
 
+/* fill_exterior = TRUE means that pattern is a surface pattern and
+ * we should extend the pattern with a solid fill from its edges.
+ * This is a bit of a hack; the alternative would be to make the
+ * surface of the surface pattern 1 pixel bigger and use CAIRO_EXTEND_PAD.
+ */
+static void
+paint_shadow_pattern_to_cairo_context (StShadow *shadow_spec,
+                                       cairo_pattern_t *pattern,
+                                       gboolean         fill_exterior,
+                                       cairo_t         *cr,
+                                       cairo_path_t    *interior_path,
+                                       cairo_path_t    *outline_path)
+{
+  /* If there are borders, clip the shadow to the interior
+   * of the borders; if there is a visible outline, clip the shadow to
+   * that outline
+   */
+  cairo_path_t *path = (interior_path != NULL) ? interior_path : outline_path;
+  double x1, x2, y1, y2;
+
+  /* fill_exterior only makes sense if we're clipping the shadow - filling
+   * to the edges of the surface would be silly */
+  g_assert (!(fill_exterior && path == NULL));
+
+  cairo_save (cr);
+  if (path != NULL)
+    {
+      cairo_append_path (cr, path);
+
+      /* There's no way to invert a path in cairo, so we need bounds for
+       * the area we are drawing in order to create the "exterior" region.
+       * Pixel align to hit fast paths.
+       */
+      if (fill_exterior)
+        {
+          cairo_path_extents (cr, &x1, &y1, &x2, &y2);
+          x1 = floor (x1);
+          y1 = floor (y1);
+          x2 = ceil (x2);
+          y2 = ceil (y2);
+        }
+
+      cairo_clip (cr);
+    }
+
+  cairo_set_source_rgba (cr,
+                         shadow_spec->color.red / 255.0,
+                         shadow_spec->color.green / 255.0,
+                         shadow_spec->color.blue / 255.0,
+                         shadow_spec->color.alpha / 255.0);
+  if (fill_exterior)
+    {
+      cairo_surface_t *surface;
+      int width, height;
+      cairo_matrix_t matrix;
+
+      cairo_save (cr);
+
+      /* Start with a rectangle enclosing the bounds of the clipped
+       * region */
+      cairo_rectangle (cr, x1, y1, x2 - x1, y2 - y1);
+
+      /* Then subtract out the bounds of the surface in the surface
+       * pattern; we transform the context by the inverse of the
+       * pattern matrix to get to surface coordinates */
+      cairo_pattern_get_surface (pattern, &surface);
+      width = cairo_image_surface_get_width  (surface);
+      height = cairo_image_surface_get_height (surface);
+
+      cairo_pattern_get_matrix (pattern, &matrix);
+      cairo_matrix_invert (&matrix);
+      cairo_transform (cr, &matrix);
+
+      cairo_rectangle (cr, 0, height, width, - height);
+      cairo_fill (cr);
+
+      cairo_restore (cr);
+    }
+
+  cairo_mask (cr, pattern);
+  cairo_restore (cr);
+}
+
 static void
 paint_background_image_shadow_to_cairo_context (StThemeNode     *node,
                                                 StShadow        *shadow_spec,
@@ -661,52 +793,128 @@ paint_background_image_shadow_to_cairo_context (StThemeNode     *node,
                                                         pattern);
     }
 
-  /* Stamp the shadow pattern out in the appropriate color
-   * in a new layer
-   */
-  cairo_push_group (cr);
-  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-  cairo_paint (cr);
-
-  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-  cairo_set_source_rgba (cr,
-                         shadow_spec->color.red / 255.0,
-                         shadow_spec->color.green / 255.0,
-                         shadow_spec->color.blue / 255.0,
-                         shadow_spec->color.alpha / 255.0);
-  cairo_paint (cr);
-
-  cairo_set_operator (cr, CAIRO_OPERATOR_DEST_IN);
-
-  cairo_set_source (cr, shadow_pattern);
-  cairo_paint (cr);
+  paint_shadow_pattern_to_cairo_context (shadow_spec,
+                                         shadow_pattern, FALSE,
+                                         cr,
+                                         interior_path,
+                                         outline_path);
   cairo_pattern_destroy (shadow_pattern);
+}
 
-  cairo_pop_group_to_source (cr);
+/* gets the extents of a cairo_path_t; slightly inefficient, but much simpler than
+ * computing from the raw path data */
+static void
+path_extents (cairo_path_t *path,
+              double       *x1,
+              double       *y1,
+              double       *x2,
+              double       *y2)
 
-  /* mask and merge the shadow
+{
+  cairo_surface_t *dummy = cairo_image_surface_create (CAIRO_FORMAT_A8, 1, 1);
+  cairo_t *cr = cairo_create (dummy);
+
+  cairo_append_path (cr, path);
+  cairo_path_extents (cr, x1, y1, x2, y2);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (dummy);
+}
+
+static void
+paint_inset_box_shadow_to_cairo_context (StThemeNode     *node,
+                                         StShadow        *shadow_spec,
+                                         cairo_t         *cr,
+                                         cairo_path_t    *shadow_outline)
+{
+  cairo_surface_t *shadow_surface;
+  cairo_pattern_t *shadow_pattern;
+  double extents_x1, extents_y1, extents_x2, extents_y2;
+  double shrunk_extents_x1, shrunk_extents_y1, shrunk_extents_x2, shrunk_extents_y2;
+  gboolean fill_exterior;
+
+  g_assert (shadow_spec != NULL);
+  g_assert (shadow_outline != NULL);
+
+  /* Create the pattern used to create the inset shadow; as the shadow
+   * should be drawn as if everything outside the outline was opaque,
+   * we use a temporary surface to draw the background as a solid shape,
+   * which is inverted when creating the shadow pattern.
    */
-  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-  cairo_save (cr);
-  if (interior_path != NULL)
+
+  /* First we need to find the size of the temporary surface
+   */
+  path_extents (shadow_outline,
+                &extents_x1, &extents_y1, &extents_x2, &extents_y2);
+
+  /* Shrink the extents by the spread, and offset */
+  shrunk_extents_x1 = extents_x1 + shadow_spec->xoffset + shadow_spec->spread;
+  shrunk_extents_y1 = extents_y1 + shadow_spec->yoffset + shadow_spec->spread;
+  shrunk_extents_x2 = extents_x2 + shadow_spec->xoffset - shadow_spec->spread;
+  shrunk_extents_y2 = extents_y2 + shadow_spec->yoffset - shadow_spec->spread;
+
+  if (shrunk_extents_x1 >= shrunk_extents_x2 || shrunk_extents_y1 >= shrunk_extents_x2)
     {
-      /* If there are borders, clip the shadow to the interior
-       * of the borders
-       */
-      cairo_append_path (cr, interior_path);
-      cairo_clip (cr);
+      /* Shadow occupies entire area within border */
+      shadow_pattern = cairo_pattern_create_rgb (0., 0., 0.);
+      fill_exterior = FALSE;
     }
-  else if (outline_path != NULL)
+  else
     {
-      /* If there is a visible outline, clip the shadow to
-       * that outline
-       */
-      cairo_append_path (cr, outline_path);
-      cairo_clip (cr);
+      /* Bounds of temporary surface */
+      int surface_x = floor (shrunk_extents_x1);
+      int surface_y = floor (shrunk_extents_y1);
+      int surface_width = ceil (shrunk_extents_x2) - surface_x;
+      int surface_height = ceil (shrunk_extents_y2) - surface_y;
+
+      /* Center of the original path */
+      double x_center = (extents_x1 + extents_x2) / 2;
+      double y_center = (extents_y1 + extents_y2) / 2;
+
+      cairo_pattern_t *pattern;
+      cairo_t *temp_cr;
+      cairo_matrix_t matrix;
+
+      shadow_surface = cairo_image_surface_create (CAIRO_FORMAT_A8, surface_width, surface_height);
+      temp_cr = cairo_create (shadow_surface);
+
+      /* Match the coordinates in the temporary context to the parent context */
+      cairo_translate (temp_cr, - surface_x, - surface_y);
+
+      /* Shadow offset */
+      cairo_translate (temp_cr, shadow_spec->xoffset, shadow_spec->yoffset);
+
+      /* Scale the path around the center to match the shrunk bounds */
+      cairo_translate (temp_cr, x_center, y_center);
+      cairo_scale (temp_cr,
+                   (shrunk_extents_x2 - shrunk_extents_x1) / (extents_x2 - extents_x1),
+                   (shrunk_extents_y2 - shrunk_extents_y1) / (extents_y2 - extents_y1));
+      cairo_translate (temp_cr, - x_center, - y_center);
+
+      cairo_append_path (temp_cr, shadow_outline);
+      cairo_fill (temp_cr);
+      cairo_destroy (temp_cr);
+
+      pattern = cairo_pattern_create_for_surface (shadow_surface);
+      cairo_surface_destroy (shadow_surface);
+
+      /* The pattern needs to be offset back to coordinates in the parent context */
+      cairo_matrix_init_translate (&matrix, - surface_x, - surface_y);
+      cairo_pattern_set_matrix (pattern, &matrix);
+
+      shadow_pattern = _st_create_shadow_cairo_pattern (shadow_spec, pattern);
+      fill_exterior = TRUE;
+
+      cairo_pattern_destroy (pattern);
     }
 
-  cairo_paint (cr);
-  cairo_restore (cr);
+  paint_shadow_pattern_to_cairo_context (shadow_spec,
+                                         shadow_pattern, fill_exterior,
+                                         cr,
+                                         shadow_outline,
+                                         NULL);
+
+  cairo_pattern_destroy (shadow_pattern);
 }
 
 /* In order for borders to be smoothly blended with non-solid backgrounds,
@@ -714,17 +922,21 @@ paint_background_image_shadow_to_cairo_context (StThemeNode     *node,
  * cases (gradients, background images, etc).
  */
 static CoglHandle
-st_theme_node_render_background_with_border (StThemeNode *node)
+st_theme_node_prerender_background (StThemeNode *node)
 {
   StBorderImage *border_image;
   CoglHandle texture;
-  int radius[4], i;
+  guint radius[4];
+  int i;
   cairo_t *cr;
   cairo_surface_t *surface;
   StShadow *shadow_spec;
+  StShadow *box_shadow_spec;
   cairo_pattern_t *pattern = NULL;
   cairo_path_t *outline_path = NULL;
   gboolean draw_solid_background = TRUE;
+  gboolean background_is_translucent;
+  gboolean interior_dirty;
   gboolean draw_background_image_shadow = FALSE;
   gboolean has_visible_outline;
   ClutterColor border_color;
@@ -738,6 +950,7 @@ st_theme_node_render_background_with_border (StThemeNode *node)
   border_image = st_theme_node_get_border_image (node);
 
   shadow_spec = st_theme_node_get_background_image_shadow (node);
+  box_shadow_spec = st_theme_node_get_box_shadow (node);
 
   actor_box.x1 = 0;
   actor_box.x2 = node->alloc_width;
@@ -748,7 +961,7 @@ st_theme_node_render_background_with_border (StThemeNode *node)
    * may need to create an image bigger than the nodes
    * allocation
    */
-  st_theme_node_get_paint_box (node, &actor_box, &paint_box);
+  st_theme_node_get_background_paint_box (node, &actor_box, &paint_box);
 
   /* translate the boxes so the paint box is at 0,0
   */
@@ -765,6 +978,12 @@ st_theme_node_render_background_with_border (StThemeNode *node)
   rowstride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32,
                                              paint_box.x2 - paint_box.x1);
   data = g_new0 (guchar, (paint_box.y2 - paint_box.y1) * rowstride);
+
+  /* We zero initialize the destination memory, so it's fully transparent
+   * by default.
+   */
+  interior_dirty = FALSE;
+
   surface = cairo_image_surface_create_for_data (data,
                                                  CAIRO_FORMAT_ARGB32,
                                                  paint_box.x2 - paint_box.x1,
@@ -775,12 +994,10 @@ st_theme_node_render_background_with_border (StThemeNode *node)
   /* TODO - support non-uniform border colors */
   get_arbitrary_border_color (node, &border_color);
 
-  for (i = 0; i < 4; i++)
-    {
-      border_width[i] = st_theme_node_get_border_width (node, i);
+  st_theme_node_reduce_border_radius (node, radius);
 
-      radius[i] = st_theme_node_get_border_radius (node, i);
-    }
+  for (i = 0; i < 4; i++)
+    border_width[i] = st_theme_node_get_border_width (node, i);
 
   /* Note we don't support translucent background images on top
    * of gradients. It's strictly either/or.
@@ -789,6 +1006,17 @@ st_theme_node_render_background_with_border (StThemeNode *node)
     {
       pattern = create_cairo_pattern_of_background_gradient (node);
       draw_solid_background = FALSE;
+
+      /* If the gradient has any translucent areas, we need to
+       * erase the interior region before drawing, so that we show
+       * what's actually under the gradient and not whatever is
+       * left over from filling the border, etc.
+       */
+      if (node->background_color.alpha < 255 ||
+          node->background_gradient_end.alpha < 255)
+        background_is_translucent = TRUE;
+      else
+        background_is_translucent = FALSE;
     }
   else
     {
@@ -803,11 +1031,25 @@ st_theme_node_render_background_with_border (StThemeNode *node)
           if (shadow_spec && pattern != NULL)
             draw_background_image_shadow = TRUE;
         }
+
+      /* We never need to clear the interior region before drawing the
+       * background image, because it either always fills the entire area
+       * opaquely, or we draw the solid background behind it.
+       */
+      background_is_translucent = FALSE;
     }
 
   if (pattern == NULL)
     draw_solid_background = TRUE;
 
+  /* drawing the solid background implicitly clears the interior
+   * region, so if we're going to draw a solid background before drawing
+   * the background pattern, then we don't need to bother also clearing the
+   * background region.
+   */
+  if (draw_solid_background)
+    background_is_translucent = FALSE;
+
   has_visible_outline = st_theme_node_has_visible_outline (node);
 
   /* Create a path for the background's outline first */
@@ -858,6 +1100,11 @@ st_theme_node_render_background_with_border (StThemeNode *node)
                              border_color.alpha / 255.);
       cairo_fill (cr);
 
+      /* We were sloppy when filling in the border, and now the interior
+       * is filled with the border color, too.
+       */
+      interior_dirty = TRUE;
+
       if (radius[ST_CORNER_TOPLEFT] > MAX(border_width[ST_SIDE_TOP],
                                           border_width[ST_SIDE_LEFT]))
         elliptical_arc (cr,
@@ -937,14 +1184,24 @@ st_theme_node_render_background_with_border (StThemeNode *node)
       cairo_append_path (cr, outline_path);
     }
 
+  if (interior_dirty && background_is_translucent)
+    {
+      cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+      cairo_fill_preserve (cr);
+      cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+    }
+
   if (draw_solid_background)
     {
+      cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+
       cairo_set_source_rgba (cr,
                              node->background_color.red / 255.,
                              node->background_color.green / 255.,
                              node->background_color.blue / 255.,
                              node->background_color.alpha / 255.);
       cairo_fill_preserve (cr);
+      cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     }
 
   if (draw_background_image_shadow)
@@ -971,6 +1228,15 @@ st_theme_node_render_background_with_border (StThemeNode *node)
       cairo_pattern_destroy (pattern);
     }
 
+  if (box_shadow_spec && box_shadow_spec->inset)
+    {
+      paint_inset_box_shadow_to_cairo_context (node,
+                                               box_shadow_spec,
+                                               cr,
+                                               interior_path ? interior_path
+                                                             : outline_path);
+    }
+
   if (outline_path != NULL)
     cairo_path_destroy (outline_path);
 
@@ -980,14 +1246,7 @@ st_theme_node_render_background_with_border (StThemeNode *node)
   texture = cogl_texture_new_from_data (paint_box.x2 - paint_box.x1,
                                         paint_box.y2 - paint_box.y1,
                                         COGL_TEXTURE_NONE,
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-                                        COGL_PIXEL_FORMAT_BGRA_8888_PRE,
-#elif G_BYTE_ORDER == G_BIG_ENDIAN
-                                        COGL_PIXEL_FORMAT_ARGB_8888_PRE,
-#else
-                                        COGL_PIXEL_FORMAT_ANY,
-#error unknown endianness type
-#endif
+                                        CLUTTER_CAIRO_FORMAT_ARGB32,
                                         COGL_PIXEL_FORMAT_ANY,
                                         rowstride,
                                         data);
@@ -1059,6 +1318,8 @@ st_theme_node_render_resources (StThemeNode   *node,
   StBorderImage *border_image;
   gboolean has_border;
   gboolean has_border_radius;
+  gboolean has_inset_box_shadow;
+  gboolean has_large_corners;
   StShadow *box_shadow_spec;
   StShadow *background_image_shadow_spec;
   const char *background_image;
@@ -1078,6 +1339,7 @@ st_theme_node_render_resources (StThemeNode   *node,
   _st_theme_node_ensure_geometry (node);
 
   box_shadow_spec = st_theme_node_get_box_shadow (node);
+  has_inset_box_shadow = box_shadow_spec && box_shadow_spec->inset;
 
   if (node->border_width[ST_SIDE_TOP] > 0 ||
       node->border_width[ST_SIDE_LEFT] > 0 ||
@@ -1095,6 +1357,27 @@ st_theme_node_render_resources (StThemeNode   *node,
   else
     has_border_radius = FALSE;
 
+  /* The cogl code pads each corner to the maximum border radius,
+   * which results in overlapping corner areas if the radius
+   * exceeds the actor's halfsize, causing rendering errors.
+   * Fall back to cairo in these cases. */
+  has_large_corners = FALSE;
+
+  if (has_border_radius) {
+    guint border_radius[4];
+    int corner;
+
+    st_theme_node_reduce_border_radius (node, border_radius);
+
+    for (corner = 0; corner < 4; corner ++) {
+      if (border_radius[corner] * 2 > height ||
+          border_radius[corner] * 2 > width) {
+        has_large_corners = TRUE;
+        break;
+      }
+    }
+  }
+
   /* Load referenced images from disk and draw anything we need with cairo now */
   background_image = st_theme_node_get_background_image (node);
   border_image = st_theme_node_get_border_image (node);
@@ -1115,22 +1398,25 @@ st_theme_node_render_resources (StThemeNode   *node,
 
   /* Use cairo to prerender the node if there is a gradient, or
    * background image with borders and/or rounded corners,
-   * since we can't do those things easily with cogl.
+   * or large corners, since we can't do those things
+   * easily with cogl.
    *
    * FIXME: if we could figure out ahead of time that a
    * background image won't overlap with the node borders,
    * then we could use cogl for that case.
    */
   if ((node->background_gradient_type != ST_GRADIENT_NONE)
-      || (background_image && (has_border || has_border_radius)))
-    node->prerendered_texture = st_theme_node_render_background_with_border (node);
+      || (has_inset_box_shadow && (has_border || node->background_color.alpha > 0))
+      || (background_image && (has_border || has_border_radius))
+      || has_large_corners)
+    node->prerendered_texture = st_theme_node_prerender_background (node);
 
   if (node->prerendered_texture)
     node->prerendered_material = _st_create_texture_material (node->prerendered_texture);
   else
     node->prerendered_material = COGL_INVALID_HANDLE;
 
-  if (box_shadow_spec)
+  if (box_shadow_spec && !has_inset_box_shadow)
     {
       if (node->border_slices_texture != COGL_INVALID_HANDLE)
         node->box_shadow_material = _st_create_shadow_material (box_shadow_spec,
@@ -1236,6 +1522,7 @@ st_theme_node_paint_borders (StThemeNode           *node,
 {
   float width, height;
   int border_width[4];
+  guint border_radius[4];
   int max_border_radius = 0;
   int max_width_radius[4];
   int corner_id, side_id;
@@ -1251,6 +1538,8 @@ st_theme_node_paint_borders (StThemeNode           *node,
   for (side_id = 0; side_id < 4; side_id++)
     border_width[side_id] = st_theme_node_get_border_width(node, side_id);
 
+  st_theme_node_reduce_border_radius (node, border_radius);
+
   for (corner_id = 0; corner_id < 4; corner_id++)
     {
       guint border_width_1, border_width_2;
@@ -1258,10 +1547,10 @@ st_theme_node_paint_borders (StThemeNode           *node,
       st_theme_node_get_corner_border_widths (node, corner_id,
                                               &border_width_1, &border_width_2);
 
-      if (node->border_radius[corner_id] > max_border_radius)
-        max_border_radius = node->border_radius[corner_id];
+      if (border_radius[corner_id] > max_border_radius)
+        max_border_radius = border_radius[corner_id];
       max_width_radius[corner_id] = MAX(MAX(border_width_1, border_width_2),
-                                        node->border_radius[corner_id]);
+                                        border_radius[corner_id]);
     }
 
   /* borders */
@@ -1285,8 +1574,8 @@ st_theme_node_paint_borders (StThemeNode           *node,
                                     alpha);
 
           /* NORTH */
-          skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
-          skip_corner_2 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
+          skip_corner_1 = border_radius[ST_CORNER_TOPLEFT] > 0;
+          skip_corner_2 = border_radius[ST_CORNER_TOPRIGHT] > 0;
 
           x1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT] : 0;
           y1 = 0;
@@ -1295,8 +1584,8 @@ st_theme_node_paint_borders (StThemeNode           *node,
           cogl_rectangle (x1, y1, x2, y2);
 
           /* EAST */
-          skip_corner_1 = node->border_radius[ST_CORNER_TOPRIGHT] > 0;
-          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
+          skip_corner_1 = border_radius[ST_CORNER_TOPRIGHT] > 0;
+          skip_corner_2 = border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
 
           x1 = width - border_width[ST_SIDE_RIGHT];
           y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPRIGHT]
@@ -1307,8 +1596,8 @@ st_theme_node_paint_borders (StThemeNode           *node,
           cogl_rectangle (x1, y1, x2, y2);
 
           /* SOUTH */
-          skip_corner_1 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
-          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
+          skip_corner_1 = border_radius[ST_CORNER_BOTTOMLEFT] > 0;
+          skip_corner_2 = border_radius[ST_CORNER_BOTTOMRIGHT] > 0;
 
           x1 = skip_corner_1 ? max_width_radius[ST_CORNER_BOTTOMLEFT] : 0;
           y1 = height - border_width[ST_SIDE_BOTTOM];
@@ -1318,8 +1607,8 @@ st_theme_node_paint_borders (StThemeNode           *node,
           cogl_rectangle (x1, y1, x2, y2);
 
           /* WEST */
-          skip_corner_1 = node->border_radius[ST_CORNER_TOPLEFT] > 0;
-          skip_corner_2 = node->border_radius[ST_CORNER_BOTTOMLEFT] > 0;
+          skip_corner_1 = border_radius[ST_CORNER_TOPLEFT] > 0;
+          skip_corner_2 = border_radius[ST_CORNER_BOTTOMLEFT] > 0;
 
           x1 = 0;
           y1 = skip_corner_1 ? max_width_radius[ST_CORNER_TOPLEFT]
@@ -1389,32 +1678,32 @@ st_theme_node_paint_borders (StThemeNode           *node,
           int n_rects;
 
           /* corner texture does not need padding */
-          if (max_border_radius == node->border_radius[corner_id])
+          if (max_border_radius == border_radius[corner_id])
             continue;
 
-          n_rects = node->border_radius[corner_id] == 0 ? 1 : 2;
+          n_rects = border_radius[corner_id] == 0 ? 1 : 2;
 
           switch (corner_id)
             {
               case ST_CORNER_TOPLEFT:
                 verts[0] = border_width[ST_SIDE_LEFT];
-                verts[1] = MAX(node->border_radius[corner_id],
+                verts[1] = MAX(border_radius[corner_id],
                                border_width[ST_SIDE_TOP]);
                 verts[2] = max_border_radius;
                 verts[3] = max_border_radius;
                 if (n_rects == 2)
                   {
-                    verts[4] = MAX(node->border_radius[corner_id],
+                    verts[4] = MAX(border_radius[corner_id],
                                    border_width[ST_SIDE_LEFT]);
                     verts[5] = border_width[ST_SIDE_TOP];
                     verts[6] = max_border_radius;
-                    verts[7] = MAX(node->border_radius[corner_id],
+                    verts[7] = MAX(border_radius[corner_id],
                                    border_width[ST_SIDE_TOP]);
                   }
                 break;
               case ST_CORNER_TOPRIGHT:
                 verts[0] = width - max_border_radius;
-                verts[1] = MAX(node->border_radius[corner_id],
+                verts[1] = MAX(border_radius[corner_id],
                                border_width[ST_SIDE_TOP]);
                 verts[2] = width - border_width[ST_SIDE_RIGHT];
                 verts[3] = max_border_radius;
@@ -1422,9 +1711,9 @@ st_theme_node_paint_borders (StThemeNode           *node,
                   {
                     verts[4] = width - max_border_radius;
                     verts[5] = border_width[ST_SIDE_TOP];
-                    verts[6] = width - MAX(node->border_radius[corner_id],
+                    verts[6] = width - MAX(border_radius[corner_id],
                                            border_width[ST_SIDE_RIGHT]);
-                    verts[7] = MAX(node->border_radius[corner_id],
+                    verts[7] = MAX(border_radius[corner_id],
                                    border_width[ST_SIDE_TOP]);
                   }
                 break;
@@ -1432,14 +1721,14 @@ st_theme_node_paint_borders (StThemeNode           *node,
                 verts[0] = width - max_border_radius;
                 verts[1] = height - max_border_radius;
                 verts[2] = width - border_width[ST_SIDE_RIGHT];
-                verts[3] = height - MAX(node->border_radius[corner_id],
+                verts[3] = height - MAX(border_radius[corner_id],
                                         border_width[ST_SIDE_BOTTOM]);
                 if (n_rects == 2)
                   {
                     verts[4] = width - max_border_radius;
-                    verts[5] = height - MAX(node->border_radius[corner_id],
+                    verts[5] = height - MAX(border_radius[corner_id],
                                             border_width[ST_SIDE_BOTTOM]);
-                    verts[6] = width - MAX(node->border_radius[corner_id],
+                    verts[6] = width - MAX(border_radius[corner_id],
                                            border_width[ST_SIDE_RIGHT]);
                     verts[7] = height - border_width[ST_SIDE_BOTTOM];
                   }
@@ -1448,13 +1737,13 @@ st_theme_node_paint_borders (StThemeNode           *node,
                 verts[0] = border_width[ST_SIDE_LEFT];
                 verts[1] = height - max_border_radius;
                 verts[2] = max_border_radius;
-                verts[3] = height - MAX(node->border_radius[corner_id],
+                verts[3] = height - MAX(border_radius[corner_id],
                                         border_width[ST_SIDE_BOTTOM]);
                 if (n_rects == 2)
                   {
-                    verts[4] = MAX(node->border_radius[corner_id],
+                    verts[4] = MAX(border_radius[corner_id],
                                    border_width[ST_SIDE_LEFT]);
-                    verts[5] = height - MAX(node->border_radius[corner_id],
+                    verts[5] = height - MAX(border_radius[corner_id],
                                             border_width[ST_SIDE_BOTTOM]);
                     verts[6] = max_border_radius;
                     verts[7] = height - border_width[ST_SIDE_BOTTOM];
@@ -1691,7 +1980,9 @@ st_theme_node_paint (StThemeNode           *node,
         {
           ClutterActorBox paint_box;
 
-          st_theme_node_get_paint_box (node, &allocation, &paint_box);
+          st_theme_node_get_background_paint_box (node,
+                                                  &allocation,
+                                                  &paint_box);
 
           paint_material_with_opacity (node->prerendered_material,
                                        &paint_box,
